<?php

namespace App\Exports;

use App\Models\User;
use App\Models\Deposit;
use App\Models\Gateway;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Events\BeforeSheet;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Events\BeforeExport;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\FromCollection;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Concerns\WithCustomStartCell;

class UserExport implements FromCollection, WithHeadings ,WithStyles,WithCustomStartCell, WithEvents
{
  
    
    public function collection()
    {
        $deposits = Deposit::select(['user_id','gateway_id','amount','status','currency'])->latest()->get();
        $srNo = 1;
        $data = $deposits->map(function ($deposit) use (&$srNo){
            return $this->transformOrder($deposit,$srNo++);
        });

        return new Collection($data);
    }
    protected function transformOrder($deposit,$sr)
    {
        $statusArray = [
            0=>'Pending',
            1=>'Approved',
            2=>'Rejected',
            
        ];
        return [
            'ID'=>$sr,
            'User Name' => $this->getUserName($deposit->user_id),
            'Gateway Name' => $this->getGatewayName($deposit->gateway_id),
            'Amount'=>$deposit->amount,
            'Currency'=>$deposit->currency,
            'Status'=>$statusArray[$deposit->status]
            
         
        ];
    }
    public function headings():array
    {
        return [
            'ID',
            'User Name',
            'Gateway Name ',
            'Amount',
            'Currency',
            'Status'
        ];
    }
    public function styles(Worksheet $sheet)
    {
        return [
           
            2   => ['font' => ['bold' => true]],
        ];
    }
    public function getUserName($id)
    {
        $user = User::select('name')->where('id',$id)->first();
        if(!empty($user))
        {
            return $user->name;
        }
        return null;
    }
    public function getGatewayName($id)
    {
        $gateway = Gateway::select('name')->where('id',$id)->first();
        if(!empty($gateway))
        {
            return $gateway->name;
        }
        return null;
    }
    public function startCell(): string
    {
        // Data will start from cell A2
        return 'A2';
    }

    public function registerEvents(): array
    {
        return [
            BeforeSheet::class => function (BeforeSheet $event) {
                $sheet = $event->sheet->getDelegate();
                $sheet->mergeCells('A1:F1'); // Adjust column range as needed
                $sheet->setCellValue('A1', 'User Deposit Report');
                $sheet->getStyle('A1')->getFont()->setBold(true)->setSize(25);
                $sheet->getStyle('A1')->getAlignment()->setHorizontal('center');
                $sheet->getRowDimension('1')->setRowHeight(30); // Set row height
                foreach (range('A', 'H') as $columnID) {
                    $sheet->getColumnDimension($columnID)->setAutoSize(true); // Adjust column width
                }
            },
        ];
    }
}
