<?php

use Carbon\Carbon;
use GuzzleHttp\Client;
use App\Models\Setting;
use Mockery\Expectation;
use App\Models\Transaction;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use Kreait\Firebase\Factory;
use Illuminate\Support\Facades\File;
use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Firebase\Messaging\Notification;
use Illuminate\Validation\ValidationException;

function getMedia($mediapath,$media_type="")
{
    if(!empty($mediapath))
    {
        return asset($mediapath);
    }
    else{
        if($media_type=='user_profile')
        {
            $randNum = rand(1, 4);
            $mediaPath = "uploads/placeholder/avatar-".$randNum.".png";
            return asset($mediaPath);
        }
        elseif($media_type=='cover' || $media_type=="")
        {
            return asset('uploads/placeholder/img.jpg');
        }
    }
}
function storeMedia($media, $type, $photo_or_video = "photo") 
{
    // Retrieve settings from config file or .env
    $maxSize = config('app.max_upload_size', 50); // Maximum file size in MB
    $allowedTypes = config('app.allowed_file_types', 'jpg,jpeg,png,gif,mp4,avi,JPG,JPEG,PNG,GIF,MP4,AVI'); // Allowed file types, comma-separated

    // Base directory for photos and videos
    $baseDir = 'uploads/';

    // Validate the media type
    if (!in_array($photo_or_video, ['photo', 'video'])) {
        session()->flash('error', "Invalid media type. Expected 'photo' or 'video'.");
        throw new InvalidArgumentException("Invalid media type. Expected 'photo' or 'video'.");
    }

    // Determine the subdirectory based on the type
    $validTypes = [
        'user_profile',
        'website_logo',
        'favicon',
        'dashboard_icon',
        'gateway_logo',
        'story',
        'story_thumbnail',
        'gift_image'
    ];
    if (!in_array($type, $validTypes)) {
        session()->flash('error', 'Invalid Type');
            throw new InvalidArgumentException("Invalid type");
        }

    $subDir = $photo_or_video . 's/' . $type;

    // Construct the final directory with date
    $dir = $baseDir . $subDir . '/' . now()->format('Y') . '/' . now()->format('m');

    // Check file size
    if ($media->getSize() > $maxSize * 1024 * 1024) {
    session()->flash('error', "Maximum Size of file is {$maxSize} ");

        throw ValidationException::withMessages(['File size' => 'File size is too large. Maximum size allowed is ' . $maxSize . 'MB']);
    }
    $prefix = null;
    $extension = $media->getClientOriginalExtension();
    // Check file type
    $allowedTypesArray = explode(',', $allowedTypes);
    if (!in_array($extension, $allowedTypesArray)) {
        session()->flash('error', 'File type not allowed. Allowed types: ' . $allowedTypes);
        throw ValidationException::withMessages(['File type' => 'File type not allowed. Allowed types: ' . $allowedTypes]);
    }
    
    $originalName = pathinfo($media->getClientOriginalName(), PATHINFO_FILENAME);
    $sanitizedFileName = Str::slug($originalName);  // Convert to URL-friendly format (e.g., my-file-name)

    // Ensure a unique file name by appending a timestamp and optionally a prefix
    $uniqueFileName = $prefix ? $prefix . '-' : '';
    $uniqueFileName .= $sanitizedFileName . '-'. uniqid().'.'.$extension;
    $destinationPath = public_path($dir);
    $newName = $dir.'/'.$uniqueFileName;
    $media->move($destinationPath, $newName);

    return $newName;
}
function get_setting($key)
{
    $settingvalue = Setting::select('value')->where('name',$key)->first(); 
    return $settingvalue->value ?? null;
}
function getUserWallet($user_id)
{
    $creditSum = Transaction::where(['user_id'=>$user_id,'flag'=>'C'])->sum('amount');
    $debitSum = Transaction::where(['user_id'=>$user_id,'flag'=>'D'])->sum('amount');
    return $creditSum-$debitSum;
}

function getUserTimezone()
{
    // Check if the user is authenticated and has a timezone set
    if (auth()->check() && !empty(auth()->user()->time_zone)) {
        return auth()->user()->time_zone;
    }

    // Attempt to get the timezone from the request header
    $headerTimezone = request()->header('X-Timezone');
    if (!empty($headerTimezone)) {
        return $headerTimezone;
    }
    return 'Asia/Karachi';

    // Default timezone if user is not authenticated, no timezone is set, and it's not provided in the header
}
function formatUserSpecificTime($datetime)
{
    return Carbon::createFromFormat('Y-m-d H:i:s', $datetime, 'UTC')
        ->timezone(getUserTimezone())
        ->toDateTimeString();
}
function convertTimeToUTC($datetime)
{
    $localTime = new DateTime($datetime, new DateTimeZone(getUserTimeZone()));

        // Convert to UTC
        $localTime->setTimezone(new DateTimeZone('UTC'));

        // Print the UTC time
        return  $localTime->format('Y-m-d H:i:s');
}
function formatDateTime($dateTime) {
    $now = new DateTime();
    $tomorrow = new DateTime('tomorrow');
    $dateTimeObj = new DateTime($dateTime);
    if ($dateTimeObj->format('Y-m-d') == $now->format('Y-m-d')) {
        return 'Today, ' . $dateTimeObj->format('h:i A');
    } elseif ($dateTimeObj->format('Y-m-d') == $tomorrow->format('Y-m-d')) {
        return 'Tomorrow, ' . $dateTimeObj->format('h:i A');
    } else {
        return $dateTimeObj->format('Y-m-d h:i A');
    }
}
function sendPushNotificaiton($username,$device,$profile_image,$text,$title,$post_id=0)
{
    // Create a Notification instance
  if($device !='' || !empty($device))
  {
        $isSingle = !is_array($device); 
        $body = $username." ".$text;
        $notification = Notification::create($title, $body,$post_id)
            ->withImageUrl($profile_image);

        // Get the Messaging instance
        $messaging = app('firebase.messaging');

        // Create a CloudMessage instance with the notification and the target token
        if ($isSingle) {
            try {
                $message = CloudMessage::withTarget('token', $device)
                    ->withNotification($notification);
                $response = $messaging->send($message);
            } catch (Exception  $e) {
                // Handle errors when the token is invalid or the user uninstalled the app
                if ($e->getCode() == 'not-registered' || $e->getCode() == 'invalid-registration-token') {
                    // Token is invalid or the user uninstalled the app
                    // Remove the token from your database
                    // Example: Device::where('token', $device)->delete();
                } else {
                    // Log other errors or handle them as needed
                    // Log::error($e->getMessage());
                }
            }
        } else {
            foreach ($device as $singleDevice) {
                try {
                    $message = CloudMessage::withTarget('token', $singleDevice)
                        ->withNotification($notification);
                    $response = $messaging->send($message);
                } catch (Exception $e) {
                    if ($e->getCode() == 'not-registered' || $e->getCode() == 'invalid-registration-token') {
                       
                    } else {
                        // Log other errors or handle them as needed
                        // Log::error($e->getMessage());
                    }
                }
            }
        }
  }
        

}
function checkPurchaseCode()
{
    $filePath = base_path('purchase_code.txt');
    if (!file_exists($filePath)) {
        header('Location: ' . route('getverifyPurchaseCode'));
        exit;
    }
    $purchaseCode = trim(File::get($filePath));
    if (empty($purchaseCode)) {
        header('Location: ' . route('getverifyPurchaseCode'));
        exit;
    }
    return 0;
}
