<?php

namespace App\Http\Controllers\Admin;

use App\Models\User;
use App\Models\Space;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;
use App\Http\Controllers\Controller;
use App\Models\ReportSpace;
use App\Models\SpaceMember;

class AdminSpaceController extends Controller
{
    public function index()
    {
        $data['page_title'] = 'All Spaces';
        return view('admin.spaces.index',$data);
    }
    public function list(Request $request)
    {
        if ($request->ajax()) {
            
            $data = Space::latest()->get();
       
        return DataTables::of($data)
            ->addIndexColumn()
            ->addColumn('username', function($data){
                $username = User::find($data->user_id)->username??'Unknown';
                return $username;
            })
            ->addColumn('start_time', function($data){
                $start_time = formatUserSpecificTime($data->start_time);
                return $start_time;
            })
            ->addColumn('action', function($row){
                $actionBtn = "<a href=".route('admin.spaces.details',$row)."  class='btn btn-primary btn-xs'><i class='fa fa-eye  m-1'></i>Details</a>";
                return $actionBtn;
            })
            ->rawColumns(['action'])
            ->make(true);
        }
    }
    public function details($id)
    {
        $data['page_title'] = 'Space Details';
        $data['space'] = Space::find($id);
        $spaceMembers  = SpaceMember::where('space_id',$id)->get();
        $users = [];
        if(!empty($spaceMembers))
        {
            $userModel = New User;
            foreach($spaceMembers as $spaceMember )
            {
                $userdata = $userModel->usershortInfo($spaceMember->user_id);
                if(!empty($userdata))
                {
                    $userdata['is_host'] = $spaceMember->is_host;
                    $userdata['is_cohost'] = $spaceMember->is_cohost;
                    $users[] = $userdata;
                }
            }
            $data['space_members'] = $users;
        }
        return view('admin.spaces.view',$data);
    }
    public function reportSpaces()
    {
        $data['reports'] = ReportSpace::select('report_spaces.*')->join('spaces', 'report_spaces.space_id', '=', 'spaces.id')
            ->where('spaces.status', 'active')
            ->where('report_spaces.status', 0)
        ->get();
       
        return view('admin.spaces.report',$data);
    }
    public function reportSpacesaction(Request $request)
    {
        $action  = $request->action;
        $report_id = $request->report_id;
        if($action=='approve')
        {
            $report = ReportSpace::find($report_id);
            
            Space::where('id',$report->space_id)->update(['is_reported'=>1]);
            ReportSpace::where('id',$report_id)->update(['status'=>1]);
            ReportSpace::where(['space_id'=>$report->space_id,'status'=>0])->delete();        
            session()->flash('success', 'Report Approved Successfully');
            return back();
        }
        else
        {
            ReportSpace::where(['id'=>$report_id])->delete();
            session()->flash('success', 'Report rejected Successfully');
            return back();
        }
    }
}
