<?php

namespace App\Http\Controllers\Admin;

use App\Models\User;
use App\Models\Space;
use App\Models\Deposit;
use App\Models\ReportUser;
use App\Exports\UserExport;
use Illuminate\Http\Request;
use App\Exports\DepositsExport;
use Yajra\DataTables\DataTables;
use Spatie\Permission\Models\Role;
use App\Http\Controllers\Controller;
use Maatwebsite\Excel\Facades\Excel;

class ManageUserController extends Controller
{
    public function index()
    {
     
        $data['page_title'] = 'All Users';
        return view('admin.users.index',$data);
    }
    public function userlist(Request $request)
    {
        if ($request->ajax()) {
            
            $data = User::latest()->get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('roles', function($row) {
                    $roles = $row->getRoleNames(); // Get the roles assigned to the user
                    $roleBadges = $roles->map(function($role) {
                        return "<span class='btn btn-xs  m-1' style='background-color:#7f7f7f;color:white;'>{$role}</span>";
                    })->implode(' '); // Format the roles as badges
                    return $roleBadges;
                })
                ->addColumn('action', function($row){
                    $actionBtn =  "<a href=".route('admin.users.assignroles', $row)." class='btn btn-primary btn-xs mx-1' title='Assign Role'> <i class='fas fa-user-tag'></i> Assign Role</a>";
                    $actionBtn .= "<a href='".route('admin.users.delete', $row)."' class='btn btn-danger btn-xs mx-1' title='delete' onclick=\"return confirm('Are you sure you want to delete this user?');\"> <i class='fas fa-trash'></i> Delete</a>";
                    return $actionBtn;
                })
                ->rawColumns(['roles', 'action']) // Mark the roles and action columns as raw HTML
                ->make(true);
        }
    }
    public function assignroles($id)
    {
        $data['page_title'] = 'Assign Role';
        $data['user'] = User::findOrFail($id);
        $data['roles'] = Role::all();
        return view('admin.users.assignrole',$data);
    }
    public function allowroles(Request $request,$id)
    {
        if($id!=1)
        {
            $roles = $request->input('roles');
            $user = User::find($id);
            $user->syncRoles([$roles]);
            
        }
        session()->flash('success', 'Role are synced  Successfully');
        return  redirect()->route('admin.users.index');
        
    }
    public function export()
    {
        return Excel::download(new UserExport, 'users.xlsx');
    }
    public function export2()
    {
        $deposits = Deposit::select(['id','user_id','gateway_id','amount','status','currency'])->latest()->get();
        $srNo = 1;
        $deposits = $deposits->map(function ($deposit) {
            $deposit->status_text = $this->getStatusText($deposit->status);
            return $deposit;
        });

        return Excel::download(new DepositsExport($deposits), 'deposits.pdf', \Maatwebsite\Excel\Excel::DOMPDF);
    }
    protected function getStatusText($status)
    {
        $statusArray = [
            0 => 'Pending',
            1 => 'Approved',
            2 => 'Rejected',
        ];

        return $statusArray[$status] ?? 'Unknown';
    }
    public function delete($id,Request $request)
    {
        if($id==1)
        {
            session()->flash('error', 'Unable to delete the administrator');
            return  redirect()->route('admin.users.index');
        }
        $user = User::find($id);
        if(empty($user)){
            session()->flash('error', 'User Not found');
            return  redirect()->route('admin.users.index');
        }
        $userModel = New User;
        $userModel->deleteUser($id);
        session()->flash('success', 'User is deleted successfully');
        return  redirect()->route('admin.users.index');
    }
    public function reports(Request $request)
    {
        $data['page_title'] = 'User report list';
        $data['reports'] = ReportUser::where('status',1)->get();
        return view('admin.users.reports',$data);
    }
    public function reportUserAction(Request $request)
    {
        $action  = $request->action;
        $report_id = $request->report_id;
        if($action=='approve')
        {
            $report = ReportUser::find($report_id);
            if(!empty($report))
            {
                $user = $report->toUser;
                if($user->id==1)
                {
                    session()->flash('error', 'Can not approve admin report');
                    ReportUser::where(['to_user_id'=>$report->to_user_id,'status'=>2])->delete();
                    return back();
                }
                $spaceCount = Space::where(['user_id'=>$report->to_user_id,'status'=>'active'])->count();
                if($spaceCount==0)
                {
                    $user->tokens()->delete();
                }
                $user->status = 2;
                ReportUser::where(['id'=>$report_id])->update(['status'=>2]);
                ReportUser::where(['to_user_id'=>$report->to_user_id,'status'=>1])->delete();
                $user->update();
            }
            session()->flash('success', 'Report Approved Successfully');
            return back();
        }
        else
        {
            ReportUser::where(['id'=>$report_id])->delete();
            session()->flash('success', 'Report rejected Successfully');
            return back();
        }
    }
}
