<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Role;
use App\Http\Controllers\Controller;
use Spatie\Permission\Models\Permission;

class RoleController extends Controller
{
    public function index()
    {
        $data['page_title'] = 'All Roles';
        $data['roles'] = Role::all();
        return view('admin.roles.index',$data);
    }
    public function create()
    {
        $data['page_title'] = 'Create Role';
        return view('admin.roles.create',$data);
    }
    public function store(Request $request)
    {
        $validate = $request->validate([
            'name'=>'required|unique:roles'
        ]);
        $name = $request->input('name');
        $res = Role::create(['name'=>$name]);
        if($res)
        {
            session()->flash('success', 'Role Created Successfully');
            return back();
        }
    }
    public function edit($id)
    {
        $data['page_title'] = 'Edit Role';
        $data['role'] = Role::find($id);
        return view('admin.roles.edit',$data);
    }
    public function update($id,Request $request)
    {
        $validate = $request->validate([
            'name'=>['required',Rule::unique('roles', 'name')->ignore($id)]
        ]);
        $role = Role::find($id);
        if($role->status=='yes')
        {
            $name = $request->input('name');
            $type = 'success';
            $message = "Role Updated Successfully";
        }
        else
        {
            $name = $role->name;
            $type = 'error';
            $message = "Role can not updateable";
        }
        
        $res = Role::where('id',$id)->update(['name'=>$name]);
        if($res)
        {
            session()->flash($type, $message);
            return redirect()->route('admin.roles.index');
        }
    }
    public function delete($id)
    {
        $role = Role::find($id);
        if($role->status=='yes')
        {
            $res = Role::where('id',$id)->delete();
            if($res)
            {
                session()->flash('success', 'Role deleted Successfully');
                return redirect()->route('admin.roles.index');
            }
        }
        else
        {
            session()->flash('error', 'Role is not deleteable ');
            return redirect()->route('admin.roles.index');
        }
    }
    public function assignpermission($id)
    {
        $data['page_title'] = 'Assign Permission ';
        $data['role'] = Role::find($id);
        $data['permissions'] = Permission::all();
        return view('admin.roles.assignpermission',$data);
    }
    public function allowpermissions(Request $request,$id)
    {
        $role = Role::find($id);
        $permissions = $request->input('permissions');
        $role->syncPermissions($permissions);
        session()->flash('success', 'Permission synced  Successfully');
        return  redirect()->route('admin.roles.index');
    }
}
