<?php

namespace App\Http\Controllers\Api;

use Stripe\Stripe;
use GuzzleHttp\Client;
use App\Models\Deposit;
use Stripe\StripeClient;
use Stripe\PaymentIntent;
use App\Models\Transaction;
use Illuminate\Http\Request;
use App\Services\PaystackService;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Stripe\Exception\InvalidRequestException;
require_once(base_path('vendor/stripe/stripe-php/init.php'));
class ApiDepositController extends Controller
{
    public function depositAmount(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'transaction_id' => 'required',
            'gateway_id' => 'required',
        ]);
    
        if ($validator->fails()) {
            return response()->json(['status' => '400', 'message' => $validator->errors()->first()], 200);
        }
        $user = $request->user();
        $gateway_id = $request->gateway_id;
        if ($gateway_id == 2) {
            Stripe::setApiKey(get_setting('STRIPE_SECRET_KEY'));
            $transactionId = $request->input('transaction_id');
            try {
                $data = PaymentIntent::retrieve($transactionId);

                if ($data->status == 'succeeded') {
                    $amount = $data->amount / 100;
                    $this->depositAmountInWallet($transactionId,$user->id,$amount,$gateway_id,$data->currency);
                    return response()->json(['code' => 200, 'message' => 'Amount  Deposited Successfully'], 200);
                }
                else
                {
                    print_r($data);
                }
            } catch (InvalidRequestException $e) {
                // Handle any errors
                return response()->json(['error' => $e->getMessage()], 500);
            }
        }
        elseif($gateway_id==3)
        {
            $transactionId = $request->transaction_id;
            $paystackService = app(PaystackService::class);  // Resolving PaystackService without constructor
            $transactiondata = $paystackService->getTransactionDetails($transactionId);
            if($transactiondata['status']=='success')
            {
                $amount = $transactiondata['amount'] / 100;
                $this->depositAmountInWallet($transactionId,$user->id,$amount,$gateway_id,$transactiondata['currency']);
                return response()->json(['code' => 200, 'message' => 'Amount  Deposited Successfully'], 200);
            }
        }
        elseif($gateway_id==4)
        {
            $client = new Client();
            $transaction_id = $request->transaction_id;

    
            $response = $client->request('GET', 'https://api.flutterwave.com/v3/transactions/' . $transaction_id . '/verify', [
                'headers' => [
                    'Authorization' => 'Bearer FLWSECK_TEST-7e04e7945ffb7cab570813a621bd1fc8-X',
                ],
            ]);

            $responseBody = json_decode($response->getBody(), true);

            if ($responseBody['status'] == 'success') {
                $details = $responseBody['data'];
                $amount = $details['amount'] - $details['app_fee'];
                $this->depositAmountInWallet($details['id'],$user->id,$amount,$gateway_id,$details['currency']);
                return response()->json(['code' => 200, 'message' => 'Amount  Deposited Successfully'], 200);
            } else {
                return response()->json([
                    'status' => 'error',
                    'message' => $responseBody['message']
                ]);
            }
        }

            

    }
    public function depositAmountInWallet($transactionId,$user_id,$amount,$gaetway_id,$currency)
    {
        $deposit  = new Deposit();
        $deposit->user_id = $user_id;
        $deposit->gateway_id = $gaetway_id; // 
        $deposit->amount = $amount;
        $deposit->currency = strtoupper($currency);
        $deposit->transaction_id = $transactionId;
        $deposit->status = 1;
        $deposit->save();
        $transaction = new Transaction;
        $transaction->user_id = $user_id;
        $transaction->flag = 'C';
        $transaction->amount = $amount;
        $transaction->description = 3;
        $transaction->save();
    }
    
}

