<?php

namespace App\Http\Controllers\API;

use App\Models\Package;
use App\Models\Transaction;
use Illuminate\Http\Request;
use App\Models\TransactionLog;
use App\Models\WithdrawMethod;
use App\Models\WithdrawRequest;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class ApiWithdrawRequestController extends Controller
{
    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type' => 'required',
            'amount' => 'required',
            'paypal_email' => 'required_if:type,Paypal',
            'iban' => 'required_if:type,Bank',
            'address' => 'required_if:type,Bank',
            'country' => 'required_if:type,Bank',
            'swift_code' => 'required_if:type,Bank',
            'full_name' => 'required_if:type,Bank',
        ]);
        if ($validator->fails()) {
            return response()->json([
                'code' => 400,
                'error_message' => $validator->errors()->first(),
            ]);
        }
        $user = $request->user();
        $amount =  $request->amount;
     

        $type = $request->type;
        $minimum_withdraw =  get_setting('minimum_withdraw_limit');

        $userwallet = getUserWallet($user->id);
        if($amount>$userwallet)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Insufficent Balance'
            ]);
        }
        if($amount<$minimum_withdraw)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Can not withdraw less than the '.$minimum_withdraw
            ]);
        }

        $withdraw = New WithdrawRequest();
        $withdraw->user_id = $user->id;
        $withdraw->amount = $amount;
        $withdraw->type = $type;

        if($type=="Paypal")
        {
            $withdraw->paypal_email = $request->paypal_email;
        }
        elseif($type=="Bank")
        {
            $withdraw->full_name = $request->full_name;
            $withdraw->iban = $request->iban;
            $withdraw->swift_code = $request->swift_code;
            $withdraw->country = $request->country;
            $withdraw->full_name = $request->full_name;
            $withdraw->address = $request->address;
        }
        $withdraw->save();
        
        $transaction = New Transaction;
        $transaction->user_id = $user->id;
        $transaction->amount = $amount;
        $transaction->flag = 'D';
        $transaction->description = 2;
        $transaction->save();
        $transaction = New Transaction;
        $transaction->user_id = 1;
        $transaction->amount = $amount;
        $transaction->flag = 'C';
        $transaction->description = 2;
        $transaction->save();
        $broadcastingClosure = config('broadcasting.broadcasting');
        if (is_callable($broadcastingClosure)) {
           $broadcastingClosure();  
        }
        return response()->json([
            'code' => 200,
            'message' => 'Withdraw request Created successfully'
        ]);

    }
    public function withdrawHistory(Request $request)
    {
        $user = $request->user();
        $limit = !empty($request->limit)?$request->limit:10;
        $offset = !empty($request->offset)?$request->offset:0;
     
        $withdraw_requests = WithdrawRequest::where('user_id',$user->id)->orderBy('id','desc')->limit($limit)->offset($offset)->get();
        if(!empty($withdraw_requests))
        {
            $statusArr = [
                1=>'Pending',
                2=>'Approved',
                3=>'Rejected'

            ];
            foreach($withdraw_requests as $withdraw_request)
            {
                $withdraw_request['status'] = $statusArr[$withdraw_request->status];
            }
            return response()->json([
                'code' => 200,
                'message' => 'Withdraw history fetch successfully',
                'data'=>$withdraw_requests
            ]);
        }
        return response()->json([
            'code' => 400,
            'error_message' => 'Withdraw History Not found',
            'data'=>$withdraw_requests
        ]);
    }
    public function transactionHistory(Request $request)
    {
        {
            $user = $request->user();
            $offset = !empty($request->offset)?$request->offset:0;
            $limit = !empty($request->limit)?$request->limit:10;
            $transactionHistory = Transaction::where(['user_id'=>$user->id])->orderBy('id','desc')->limit($limit)->offset($offset)->get();
            if(!empty($transactionHistory))
            {

                return response()->json([
                    'code' => 200,
                    'message' => 'Transaction History fetch successfully',
                    'data'=>$transactionHistory
                ]);
            }
            return response()->json([
                'code' => 400,
                'error_message' => 'Transaction History Not found',
            ]);
        }
    }



}
