<?php

namespace App\Http\Controllers\Api;

use Exception;
use App\Models\User;
use App\Models\Followers;
use App\Models\Notification;
use Illuminate\Http\Request;
use App\Mail\ForgotPasswordEmail;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Laravel\Sanctum\PersonalAccessToken;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{
    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required',
            'time_zone'=>'required'
        ]);

        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()], 200);
        }
        $email = $request->email;
        $password = $request->password;
        
        $user = User::where('email',$email)->first();
        if(!empty($user)){
            if($user->status!=1)
            {
                return response()->json(['code'=>400,'error_message'=>'Your account is suspended '], 200);
                
            }
            if($user->is_verify==0)
            {
                $randomCode = rand(100000,999999);
                User::where('id',$user->id)->update(['verify_code'=>$randomCode]);
                Mail::to($request->email)->send(new ForgotPasswordEmail($user, $randomCode));
                return response()->json(['code'=>200,'is_verify'=>0,'message'=>'Please check the Your Email with code of 6 digit numbers '], 200);
            }
            if ($user && password_verify($password, $user->password)) {
                Auth::login($user);
                $user = Auth::user();
                $user->tokens()->delete();

                $user->device_id    = $request->device_id;
                $user->device_type  = $request->device_type;
                $user->time_zone  = $request->time_zone;
                $user['profile_image']  = getMedia($user->profile_image,'user_profile');
            
                
                $token =  $user->createToken('MyApp')->plainTextToken;
                $user->update();
                $user['is_following']  =0;
                return response()->json(['code'=>200,'token'=>$token,'data'=>$user], 200);
             }
             else
             {
                return response()->json(['code'=>400,'error_message'=>'Invalid User Name or Password'], 200);
             }
             
        
        }
        else{
            return response()->json(['code'=>400,'error_message'=>'Invalid User Name or Password'], 200);
        }
    }
   
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email',
            'password' => 'required',
            'c_password' => 'required|same:password',
            'username' => 'required',
        ]);

        if ($validator->fails()) {
            $first_error = $validator->errors()->first();
            return response()->json(['code' => 400, 'error_message' => $first_error], 200);
        }

        $found_user = User::where('email', $request->email)->first();

        if ($found_user) {
            return $this->handleExistingUser($found_user, $request->email);
        }

        $user = $this->createNewUser($request);

        $token = $user->createToken('MyApp')->plainTextToken;

        // Queue the email sending to avoid blocking
        Mail::to($request->email)->queue(new ForgotPasswordEmail($user, $user->verify_code));

        // Handle following users
        $this->handleFollowingUsers($request->following_ids, $user);

        return response()->json([
            'code' => 200,
            'message' => 'Verification code is sent to your email address to verify your email',
            'is_verify'=>0
        ], 200);
    }

    protected function handleExistingUser($found_user, $email)
    {
        if ($found_user->is_verify == 0) {
            $randomCode = rand(100000, 999999);
            $found_user->verify_code = $randomCode;
            $found_user->update();
            Mail::to($email)->queue(new ForgotPasswordEmail($found_user, $randomCode));

            return response()->json([
                'code' => 200,
                'message' => 'Please check your email for the 6-digit code',
                'is_verify' => 0
            ], 200);
        } else {
            return response()->json(['code' => 400, 'error_message' => 'Email already taken'], 200);
        }
    }

    protected function createNewUser($request)
    {
        $user = new User;
        $user->username = $request->username;
        $user->first_name = $request->first_name;
        $user->last_name = $request->last_name;
        $user->about = $request->about;
        $user->email = $request->email;
        $user->password = Hash::make($request->password);
        $user->device_type = $request->device_type ?? null;
        $user->device_id = $request->device_id ?? null;
        $user->verify_code = rand(100000, 999999);
        $user->profile_image = "uploads/placeholder/avatar-" . rand(1, 4) . ".png";
        $user->save();

        return $user;
    }

    protected function handleFollowingUsers($followingIdString, $user)
    {
        if (empty($followingIdString)) {
            return;
        }

        $followingIdArray = explode(',', $followingIdString);
        $userdevicedata = [];

        foreach ($followingIdArray as $followingId) {
            $followinguser = User::find($followingId);
            if ($followinguser) {
                Followers::updateOrCreate([
                    'from_user_id' => $user->id,
                    'to_user_id' => $followingId
                ]);
                $followinguser->increment('follower_count');

                if ($followinguser->pause_notificaiton == 0) {
                    Notification::create([
                        'from_user_id' => $user->id,
                        'to_user_id' => $followingId,
                        'notification_type' => 'follow_user',
                        'text' => 'started following you'
                    ]);
                    $userdevicedata[] = $followinguser->device_id;
                }
            }
        }

        $user->increment('following_count', count($followingIdArray));

        // if (!empty($userdevicedata)) {
        //     sendPushNotificaiton($user->username, $userdevicedata, $user->profile_image, 'started following you');
        // }
    }

    public function resetPassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email'=>'required',
        ]);

        if ($validator->fails()) {

            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 401);
        }
        $email = $request->email;
        $found_user = User::where('email',$email)->first();
        if(!empty($found_user) || $found_user!=null)
        {
            $randomCode = rand(100000,999999);
            $found_user->verify_code = $randomCode;
            $found_user->sent_at = NOW();
            $found_user->update();
           
            try {
              
                Mail::to($email)->send(new ForgotPasswordEmail($found_user, $randomCode));
          
            } catch (Exception $e) {
              
                Log::error('Email sending error: ' . $e->getMessage());
            
                // You can also return an error response or display a message to the user
                return redirect()->back()->with('error', 'An error occurred while sending the email.');
            }
            
            return response()->json([
                'code' =>200,
                'data' =>  "Please Check Your email with code of 6 number"
            ],200);
        }
        return response()->json([
            'code' => 400,
            'error_message'=> 'Email not found'
        ],200);
    }
    public function reset_password_verified(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'verify_code'=>'required',
            'email'=>'required',
            'password'=>'required|min:8|confirmed'
        ]);

        if ($validator->fails()) {
            return response()->json(['code'=>400,'error_message'=>$validator->messages()->first()], 200);
        }
        $verify_code = $request->verify_code;
        $email = $request->email;
        $user = User::where(['email'=>$email,'verify_code'=>$verify_code])->first();
        if($user!=null)
        {
            $password = $request->password;
            $user->password = Hash::make($password);
            $user->email_verified_at = NOW();
            $user->verify_code = null;
            $user->update();

            return response()->json([
                'code' =>200,
                'message' => 'Your Password is changed '
            ]);
        }
        return response()->json([
            'code' =>400,
            'message' => 'Your Code is incorrect '
        ]);
    }
    public function verifyCode(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'verify_code'=>'required',
            'email'=>'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $verify_code = $request->verify_code;
        $email = $request->email;
        $userdata = User::where(['email'=>$email,'verify_code'=>$verify_code])->first();
        if(empty($userdata))
        {
            return response()->json(['code'=>400,'error_message'=>'Please Enter the correct Code'], 200);
        }
        $userdata->is_verify = 1;
        $userdata->verify_code = NULL;
        $userdata->sent_at = NULL;
        $userdata->update();     
        $userdata->tokens()->delete();
        
        $userdata['profile_image']  = getMedia($userdata->profile_image,'user_profile');
        $token =  $userdata->createToken('MyApp')->plainTextToken;
        
        return response()->json(['code'=>200,'token'=>$token,'data'=>$userdata], 200);
    }
    public function sendOTP(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email'=>'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $email = $request->email;
      
        $found_user = User::where('email',$email)->first();

        if(!empty($found_user) || $found_user!=null)
        {
            $randomCode = rand(100000,999999);
            $found_user->verify_code = $randomCode;
            $found_user->sent_at = NOW();
            $found_user->update();
            try {
              
                Mail::to($email)->send(new ForgotPasswordEmail($found_user, $randomCode));
          
            } catch (Exception $e) {
              
                Log::error('Email sending error: ' . $e->getMessage());
            
                // You can also return an error response or display a message to the user
                return redirect()->back()->with('error', 'An error occurred while sending the email.');
            }
            return response()->json(['code'=>200,'message'=>'Please Check Your email with code of 6 number'], 200);
        }  
        return response()->json(['code'=>400,'message'=>'Email not found'], 200);

    }
    public function logout(Request $request)
    {
        $user = $request->user();
        User::where('id',$user->id)->update(['device_id'=>'']);
        $user->tokens->each(function (PersonalAccessToken $token) {
            $token->delete();
        });
        return response()->json(['code'=>200,'message'=> 'User Logout Successfully'], 200);
    }
    public function checkUsername(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'first_name'=>'required',
            'last_name'=>'required',
            
        ]);
        if ($validator->fails()) {
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $first_name= strtolower($request->first_name);
        $last_name= strtolower($request->last_name);
        
        $username = $first_name.$last_name;
        $usernames = [];
        $checkExistUsername = User::where('username',$username)->first();
        if(empty($checkExistUsername))
        {
            $usernames[] = $username;
            $usernames[] = $last_name.$first_name;
            $usernames[] = $username.rand(100,999);
            
        }
        else
        {
            $usernames[] = $username.rand(100,999);
            $usernames[] = $last_name.$first_name;
            $usernames[] = $username.rand(100,999);
        }
        return response()->json(['code'=>200,'message'=>"Avaialable usernames fetch successfully",'data'=>$usernames], 200);
    }
    public function checkExistEmail(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email'=>'required|email',
        ]);
        if ($validator->fails()) {
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $email = $request->email;
        $userdata = User::where('email',$email)->first();
        if(!empty($userdata)){
            return response()->json([
                'code'=>200,
                'message' => 'Email already exist ',
                'data'=>true
            ]);
        }
        return response()->json([
            'code'=>200,
            'message' => 'Email does not exist can use this',
            'data'=>false
        ]);
 
    }
}
