<?php

namespace App\Http\Controllers\Api;

use App\Models\User;
use App\Models\Space;
use App\Models\Notification;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class NotificationController extends Controller
{
    public function getUnreadNotifications(Request $request)
    {
        $limit = $request->limit??10;
        $offset =$request->offset??0;
        $user = $request->user();
        $notifications = Notification::where(['to_user_id'=>$user->id,'is_read'=>0])->where('from_user_id','!=',$user->id)->limit($limit)->offset($offset)->latest()->get();
        if(!empty($notifications) && count($notifications)>0)
        {
            $userModel  = New User;
            foreach($notifications as $notification)
            {
                $notification['notifier'] = $userModel->usershortInfo($notification->from_user_id,$user->id);
            }
            return response()->json(['code' => 200, 'message' => 'Notification fetch successfully', 'data' =>$notifications], 200);
        }
        return response()->json(['code' => 200, 'message' => 'Notification not found', 'data' =>$notifications], 200);
    }
    public function getAllNotifications(Request $request)
    {
        $limit = $request->limit??10;
        $offset =$request->offset??0;
        $user = $request->user();
        $notifications = Notification::where('to_user_id',$user->id)->where('from_user_id','!=',$user->id)->limit($limit)->offset($offset)->latest()->get();
        if(!empty($notifications) && count($notifications)>0)
        {
            $userModel  = New User;
            $spaceModel = New Space;
            foreach($notifications as $notification)
            {
                $notification['notifier'] = $userModel->usershortInfo($notification->from_user_id,$user->id);
                // if($notification->space_id!=0)
                // {
                //     $space = Space::find($notification->space_id);
                //     $notification['space'] = $spaceModel->getsingleSpace($space,$user->id);
                // }
            }
            return response()->json(['code' => 200, 'message' => 'Notification fetch successfully', 'data' =>$notifications], 200);
        }
        return response()->json(['code' => 200, 'message' => 'Notification not found'], 200);
    }
    public function markAsRead(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'notification_id'=>'required'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'code' => 400,
                'error_message' => $validator->errors()->first(),
            ]);
        }
        $user = $request->user();
        $notification_id  = $request->notification_id;
        $notification = Notification::find($notification_id);
        if(empty($notification))
        {
            return response()->json(['code' => 400, 'error_message' => 'Notification not found'], 200);
        }
        if($notification->to_user_id!=$user->id)
        {
            return response()->json(['code' => 400, 'error_message' => 'UnAuthenticated'], 200);
        }
        $notification->is_read = 1;
        $notification->update();
        return response()->json(['code' => 200, 'message' => 'Notification mark as read successfully'], 200);
    }
    public function markAllAsRead(Request $request)
    {
        $user = $request->user();
        Notification::where(['to_user_id'=>$user->id,'is_read'=>0])->update(['is_read'=>1]);
        return response()->json(['code' => 200, 'message' => 'Notification mark as read successfully'], 200);
    }
    public function deleteAllNotification(Request $request)
    {
        $user = $request->user();
        Notification::where(['to_user_id'=>$user->id])->delete();
        return response()->json(['code' => 200, 'message' => 'Notification deleted successfully'], 200);
    }
    public function deleteNotitication(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'notification_id'=>'required'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'code' => 400,
                'error_message' => $validator->errors()->first(),
            ]);
        }
        $user = $request->user();
        $notification_id  = $request->notification_id;
        $notification = Notification::find($notification_id);
        if(empty($notification))
        {
            return response()->json(['code' => 400, 'error_message' => 'Notification not found'], 200);
        }
        if($notification->to_user_id!=$user->id)
        {
            return response()->json(['code' => 400, 'error_message' => 'UnAuthenticated'], 200);
        }
        $notification->delete();
        return response()->json(['code' => 200, 'message' => 'Notification deleted successfully'], 200);
    }
}
