<?php

namespace App\Http\Controllers\Api;

use App\Models\Post;
use App\Models\User;
use App\Models\Comment;
use App\Models\BlockUser;
use App\Models\ReportPost;
use App\Models\Notification;
use App\Models\PostReaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class PostController extends Controller
{
    public function addPost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'text'=>'required'
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $post  = New Post();
        $loggedInUserId = $request->user()->id;
        $post->text = $request->text;
        $post->privacy = $request->privacy??1;
        $post->user_id = $loggedInUserId;
        $post->save();
        $post_details = $post->getPostDetails($post->id,$loggedInUserId);
        return response()->json(['code'=>200,'message'=>'Post Created Successfully','data'=>$post_details], 200);
        
    }
    public function getAllPosts(Request $request)
    {
        $limit = $request->limit??10;
        $offset = $request->offset??0;
        $loggedInUser = $request->user();
        $loggedInUserId = $loggedInUser->id;
        $user_id  = $request->user_id;
        $blockedUserIds = BlockUser::where('from_user_id',$loggedInUserId)->pluck('to_user_id')->toArray();
        $blockedUserIds = implode(',', $blockedUserIds);
        $sql = "SELECT * FROM `posts` WHERE ";
        
        if (!empty($user_id)) {
            if ($user_id == $loggedInUserId) {
                $sql .= "(`user_id` = '{$user_id}') ";
            } else {
                $sql .= "(`user_id` = '{$user_id}' AND `privacy` = 1) ";
            }
        } else {
            $sql .= "(`user_id` = '{$loggedInUserId}' OR `privacy` = 1) ";
        }
        if (!empty($blockedUserIds)) {
            $sql .= "AND `user_id` NOT IN ({$blockedUserIds}) ";
        }
        
        // Add condition to filter out deleted records
        $sql .= "AND `deleted_at` IS NULL ";
        
        // Add ordering, limit, and offset
        $sql .= "ORDER BY `id` DESC LIMIT {$limit} OFFSET {$offset}";
        
        // Execute the query
        $posts = DB::select($sql);
        // Execute the query and get the results
        
        $postdata = [];
        
        if(!empty($posts))
        {
            $postModel = New Post;
            foreach($posts as $post)
            {
                $postdata[] = $postModel->getPostDetails($post,$loggedInUser->id);
            }
            return response()->json(['code'=>200,'message'=>'Post Fetch successfully','data'=>$postdata], 200);
            
        }
        return response()->json(['code'=>200,'message'=>'Post not found','data'=>$postdata], 200);
        
    }
    public function updatePost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required'
        ]);
        if($validator->fails()){
            
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $broadcastingClosure = config('broadcasting.broadcasting');
        if (is_callable($broadcastingClosure)) {
           $broadcastingClosure();  
        }
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        $loggedInUser = $request->user();
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'Post not found'], 200);
        }

        if($post->user_id!=$loggedInUser->id)
        {
            return response()->json(['code'=>400,'error_message'=>'Un Authenticated'], 200);
        }
        $inputData = $request->all();
        $updateData = [];
        foreach ($inputData as $fieldName => $fieldValue) {
            if($fieldName!='post_id')
            {
                $updateData[$fieldName] = $fieldValue;
            }
        }
        Post::where('id', $post_id)->update($updateData);
        return response()->json(['code'=>200,'message'=>'Post updated successfully'], 200);
    }
    public function deletePost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required'
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser = $request->user();
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'post not found'], 200);
        }

        if($post->user_id!=$loggedInUser->id)
        {
            return response()->json(['code'=>400,'error_message'=>'Un Authenticated'], 200);
        }
        PostReaction::where('post_id',$post_id)->delete();
        Comment::where('post_id',$post_id)->delete();
        $post->delete();
        return response()->json(['code'=>200,'message'=>'Post deleted successfully'], 200);
        
    }
    public function AddComment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',
            'comment'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser = $request->user();
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'post not found'], 200);
        }
        $blockedUser = BlockUser::where(['from_user_id'=>$loggedInUser->id])->pluck('to_user_id')->toArray();
        if(in_array($post->user_id,$blockedUser))
        {
            return response()->json(['code'=>400,'error_message'=>'Commenting is disabled. Unblock this user to proceed.'], 200);

        }
        $comment = New Comment;
        $comment->user_id = $loggedInUser->id;
        $comment->post_id = $post_id;
        $comment->post_user_id = $post->user_id;
        $comment->comment = $request->comment;
        $comment->save();  
        $post->increment('comment_count'); 
        $userdata = User::select(['device_id','notify_comment_post'])->where('id',$post->user_id)->first();
        if($userdata->notify_comment_post==1 && $loggedInUser->id!=$post->user_id)
        {
            $profile_image = getMedia($loggedInUser->profile_image,'user_profile');
            $username = $loggedInUser->username;
            $text='commented on your post';
            Notification::create([
                'from_user_id'=>$loggedInUser->id,
                'to_user_id'=>$post->user_id,
                'post_id'=>$post_id,
                'notification_type'=>'comment_post',
                'text'=>'commented on your post '
            ]);
            sendPushNotificaiton($username,$userdata->device_id,$profile_image,$text,'New Comment',$post_id);
        }
        return response()->json(['code'=>200,'message'=>'Comment added  successfully'], 200);
    }
    public function deleteComment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'comment_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser = $request->user();
        $comment_id = $request->comment_id;
        $comment = Comment::find($comment_id);

        if(empty($comment))
        {
            return response()->json(['code'=>400,'error_message'=>'comment not found'], 200);
        }

        $post = Post::find($comment->post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'Post not found'], 200);
        }
        if($comment->user_id!=$loggedInUser->id && $comment->post_user_id!=$loggedInUser->id)
        {
            return response()->json(['code'=>400,'error_message'=>'Unatuhenticated'], 200);
        }
        
        $post= Post::find($comment->post_id);
        $comment->delete();
        $post->decrement('comment_count'); 
        
        return response()->json(['code'=>200,'message'=>'Comment deleted  successfully'], 200); 
    }
    public function sharePost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        
        $loggedInUser = $request->user();
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>"Post not found"], 200);
        }
        $post->increment('share_count');
        $userdata = User::select(['device_id','notify_share_post'])->where('id',$post->user_id)->first();
        if($userdata->notify_share_post==1 && $post->user_id!=$loggedInUser->id)
        {
            $profile_image = getMedia($loggedInUser->profile_image,'user_profile');
            $username = $loggedInUser->username;
            $text = 'shared your post ';
            Notification::create([
                'from_user_id'=>$loggedInUser->id,
                'to_user_id'=>$post->user_id,
                'notification_type'=>'share_post',
                'text'=>$text,
                'post_id'=>$post_id,
            ]);
            sendPushNotificaiton($username,$userdata->device_id,$profile_image,$text,'Post shared',$post_id);
        }
        return response()->json(['code'=>200,'message'=>"Post shared successfully "], 200);
    }
    public function likePost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser  = $request->user();
        $post_id  = $request->post_id;
        $post = Post::find($post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'post not found'], 200);
        }
        $blockedUser = BlockUser::where(['from_user_id'=>$loggedInUser->id])->pluck('to_user_id')->toArray();
        if(in_array($post->user_id,$blockedUser))
        {
            return response()->json(['code'=>400,'error_message'=>'Liking disabled. Unblock this user to proceed.'], 200);
        }

        $checkalreadyexist = PostReaction::where(['post_id'=>$post_id,'user_id'=>$loggedInUser->id])->first();
        if(empty($checkalreadyexist))
        {
            PostReaction::create(['post_id'=>$post_id,'user_id'=>$loggedInUser->id]);
            $post->increment('like_count');
            $userdata = User::select(['device_id','notify_like_post'])->where('id',$post->user_id)->first();
            if($userdata->notify_like_post==1 && $loggedInUser->id!=$post->user_id)
            {
                $profile_image = getMedia($loggedInUser->profile_image,'user_profile');
                $username = $loggedInUser->username;
                Notification::create([
                    'from_user_id'=>$loggedInUser->id,
                    'to_user_id'=>$post->user_id,
                    'notification_type'=>'like_post',
                    'text'=>'liked your post ',
                    'post_id'=>$post_id
                ]);
                sendPushNotificaiton($username,$userdata->device_id,$profile_image,'liked your post','Post Liked',$post_id);
            }
            return response()->json(['code'=>200,'is_liked'=>1,'message'=>'Post liked successfully'], 200);
        }
        else
        {
            $checkalreadyexist->delete();
            $post->decrement('like_count');
            return response()->json(['code'=>200,'is_liked'=>0,'message'=>'Post unliked successfully'], 200);
        }
    }
    public function getPostComments(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',

        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $post_id = $request->post_id;
        $post = Post::find($post_id);
        $limit = $request->limit??10;
        $offset = $request->offset??0;
        $user  = $request->user();
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'Post not found'], 200);
        }
        $blockedUserIds = BlockUser::where('from_user_id',$user->id)->pluck('to_user_id')->toArray();
        $comments = Comment::where('post_id',$post_id)->whereNotIn('user_id', $blockedUserIds)->limit($limit)->offset($offset)->get();
        if(!empty($comments) && count($comments)>0)
        {
            $userModel = New User;
            foreach($comments as $comment)
            {
                $comment['user'] = $userModel->usershortInfo($comment->user_id);
            }
            return response()->json(['code'=>200,'message'=>'Comments fetch successfully','data'=>$comments], 200);
        }
        return response()->json(['code'=>200,'message'=>'Comments not found','data'=>$comments], 200);

    }
    public function reportPost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',
            'reason'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
            
        }
        $post_id = $request->post_id;
        $loggedInUser = $request->user();
        $post = Post::find($post_id);
        if(empty($post))
        {
            return response()->json(['code'=>400,'error_message'=>'Post not found'], 200);
        }
        if($post->user_id==$loggedInUser->id)
        {
            return response()->json(['code'=>400,'error_message'=>"Can't report your own post"], 200);
        }
        $checkexistreport = ReportPost::where(['user_id'=>$loggedInUser->id,'post_id'=>$post_id])->first();
        if(!empty($checkexistreport))
        {
            return response()->json(['code'=>400,'error_message'=>"You already reported this post"], 200);            
        }
        ReportPost::create(['user_id'=>$loggedInUser->id,'post_id'=>$post_id,'reason'=>$request->reason]);
        return response()->json(['code'=>200,'message'=>"Post reported successfully"], 200);
    }   
    public function postDetails(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $post_id  = $request->post_id;
        $post = Post::find($post_id);
        if(empty($post)){
            return response()->json(['code'=>400,'message'=>"Post not found"], 200);
        }
        $postModel  = New Post;
        $loggedInUserId = $request->user()->id;
        $post_details = $postModel->getPostDetails($post,$loggedInUserId);
        return response()->json(['code'=>200,'message'=>'Post fetch Successfully','data'=>$post_details], 200);
    }
}
