<?php

namespace App\Http\Controllers\Api;

use App\Models\Gift;
use App\Models\User;
use App\Models\Space;
use App\Models\Category;
use App\Models\BlockUser;
use App\Models\Followers;
use App\Models\ReportSpace;
use App\Models\SpaceMember;
use App\Models\SubCategory;
use App\Models\Transaction;
use App\Models\Notification;
use App\Models\SpaceCheckIn;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use App\Classes\AgoraDynamicKey\RtcTokenBuilder;

include ("./../app/Classes/AgoraDynamicKey/RtcTokenBuilder.php");
class SpaceController extends Controller
{
    public function createSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'title'=>'required',
            'description'=>'required',
            'is_paid'=>'required',
            'amount'=>'required_if:is_paid,1',
            'privacy'=>'required',
            'start_time'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser = $request->user();
        $status = $request->status;
        if($status=='active')
        {
            Space::where(['user_id'=>$loggedInUser->id,'status'=>'active'])->update(['status'=>'completed']);
        }
        $space = New Space;
        $space->user_id = $loggedInUser->id;                                                                                                                                                       
        $space->title = $request->title;
        $space->description = $request->description;
        $space->category_id = $request->category_id??0;
        $space->sub_category_id = $request->sub_category_id??0;
        $space->rules = $request->rules;
        $space->status = $request->status;
        if(!empty($request->is_paid))
        {
            $space->is_paid = 1;
            $space->amount = $request->amount;
        }
        else
        {
            $space->is_paid = 0;
            $space->amount = 0;
        }
        $space->privacy = $request->privacy??1;
        $space->start_time =  convertTimeToUTC($request->start_time);
        $space->agora_token_id = $request->agora_token_id;
        $space->save();
        $space_id =$space->id;
        SpaceMember::create(['user_id'=>$loggedInUser->id,'space_id'=>$space_id,'is_host'=>1]);
        $cohosts = $request->cohost_ids;
      
        if(!empty($cohosts))
        {
            $coHostArray = explode(",", $cohosts);
            $cohostfcmId = [];
            if(count($coHostArray)>0)
            {
                foreach($coHostArray as $cohost)
                {
                    $cohostdata = User::select(['id','device_id','notify_add_cohost'])->where('id',$cohost)->first();
                    if($cohost!=$loggedInUser->id && !empty($cohostdata) )
                    {
                        $spaceCheckIn = New SpaceCheckIn;
                        $spaceCheckIn->user_id = $cohost;
                        $spaceCheckIn->space_id= $space_id;
                        $spaceCheckIn->is_cohost = 1;
                        if($space->privacy==3)
                        {
                            $spaceCheckIn->is_private_space = 1;
                        }
                        $spaceCheckIn->payment_status = ($space->is_paid==1)?1:0;
                        $spaceCheckIn->save();
                        if($cohostdata->notify_add_cohost==1)
                        {
                            $text = 'Invited you as cohost in space';
                            Notification::create([
                                'from_user_id' =>$loggedInUser->id,
                                'to_user_id' =>$cohost,
                                'space_id'=>$space_id,
                                'text'=>$text,
                                'notification_type'=>'add_cohost'
                            ]);
                            $username = $loggedInUser->username;
                            $profile_image = getMedia($loggedInUser->profile_image);
                            if(!empty($cohostdata->device_id))
                            {
                                $cohostfcmId[] =  $cohostdata->device_id;
                                // array_push($cohostfcmId,$cohostdata->device_id);
                            }  
                        }
                    }

                }
            }
            // print_r($cohostfcmId);

            // if(count($cohostfcmId)>0)
            // {
            //     print_r($cohostfcmId);
            //     die;
            //     sendPushNotificaiton($username,$cohostfcmId,$profile_image,$text,'Cohost Invite');
            // }
        }
        // $memberIdString = $request->member_ids;
        // if(!empty($memberIdString) && $space->privacy==3)
        // {
        //     $memberIdArray = explode(",", $memberIdString);
        //     foreach($memberIdArray as $member)
        //     {
        //         $spaceCheckIn = New SpaceCheckIn;
        //         $spaceCheckIn->user_id = $member;
        //         $spaceCheckIn->space_id= $space_id;
        //         $spaceCheckIn->is_cohost = 0;
        //         if($space->privacy==3)
        //         {
        //             $spaceCheckIn->is_private_space = 1;
        //         }
        //         $spaceCheckIn->payment_status = 0;
        //         $spaceCheckIn->save();

        //     }
        // }

        $spaceModel = New Space;
        $spacedata = Space::find($space->id);
        $spacedetails = $spaceModel->getsingleSpace($spacedata,$loggedInUser->id);
        return response()->json(['code'=>200,'message'=>'Space is created','data'=>$spacedetails], 200);

    }  
                                                           
    public function getCategories(Request $request)
    {
        $this->checkandreversetransaction();
        $limit = $request->limit ?? 10;
        $offset = $request->offset ?? 0;
        $categories = Category::with('subcategories')->limit($limit)->offset($offset)->get();
        if(count($categories)>0)
        {
            return response()->json(['code'=>200,'message'=>'Categories fetch successfully','data'=>$categories], 200);
        }
        return response()->json(['code'=>400,'error_message'=>'Categories not found'], 200);

    }
    public function getSpaces(Request $request)
    {
        $this->checkandreversetransaction();
        $searchTitle    = $request->searchTitle; 
        $timeFilter     = $request->timeFilter;
        $categoryId     = $request->category_id;
        $subCategoryId  = $request->sub_category_id;
        
        $type = $request->type;
        $user = $request->user();
        $loggedInUserId = $user->id;
        $limit = $request->limit ?? 10;
        $offset = $request->offset ?? 0;
        $followedUserIds = Followers::where('from_user_id', $loggedInUserId)->pluck('to_user_id')->toArray();
        $blockedUserIds = BlockUser::where('from_user_id', $loggedInUserId)->pluck('to_user_id')->toArray();
        $blockedUserIdsString = !empty($blockedUserIds) ? implode(',', $blockedUserIds) : '';
        
        $query = "SELECT * FROM `spaces` WHERE ";

        if ($type != 'my') {
            // Handle non-'my' type
            $followedUserIdsString = !empty($followedUserIds) ? implode(',', $followedUserIds) : '';
            
            $query .= " (
                `privacy` = 1";
        
            if (!empty($followedUserIdsString)) {
                $query .= " OR (`privacy` = 2 AND `user_id` IN ($followedUserIdsString))";
            }
        
            $query .= " OR (`privacy` = 3 AND `id` IN (SELECT space_id FROM space_check_ins WHERE is_private_space=1 AND user_id = {$loggedInUserId})) ";
        
            $query .= " )
                AND `status` = '{$type}'
                AND `user_id` != {$loggedInUserId} ";
        
        } else {
            // Handle 'my' type
            $query .= "`user_id` = {$loggedInUserId} AND status = 'upcoming' ";
        }
        
        // Add search filter based on title
        if (!empty($searchTitle)) {
            $searchTitle = '%' . $searchTitle . '%';
            $query .= " AND `title` LIKE '{$searchTitle}'";
        }
        
        // Add time filter only if the type is 'upcoming'
        if ($type == 'upcoming') {
            $currentDate = date('Y-m-d');
            switch ($timeFilter) {
                case 'today':
                    $query .= " AND DATE(`start_time`) = '{$currentDate}'";
                    break;
                case 'tomorrow':
                    $tomorrowDate = date('Y-m-d', strtotime('+1 day'));
                    $query .= " AND DATE(`start_time`) = '{$tomorrowDate}'";
                    break;
                case 'week':
                    $weekEndDate = date('Y-m-d', strtotime('+7 days'));
                    $query .= " AND DATE(`start_time`) BETWEEN '{$currentDate}' AND '{$weekEndDate}'";
                    break;
                case 'month':
                    $monthEndDate = date('Y-m-d', strtotime('+30 days'));
                    $query .= " AND DATE(`start_time`) BETWEEN '{$currentDate}' AND '{$monthEndDate}'";
                    break;
                case 'next_week':
                    $nextWeekStartDate = date('Y-m-d', strtotime('+7 days'));
                    $nextWeekEndDate = date('Y-m-d', strtotime('+14 days'));
                    $query .= " AND DATE(`start_time`) BETWEEN '{$nextWeekStartDate}' AND '{$nextWeekEndDate}'";
                    break;
                case 'next_month':
                    $nextMonthStartDate = date('Y-m-d', strtotime('+30 days'));
                    $nextMonthEndDate = date('Y-m-d', strtotime('+60 days'));
                    $query .= " AND DATE(`start_time`) BETWEEN '{$nextMonthStartDate}' AND '{$nextMonthEndDate}'";
                    break;
                case 'all':
                default:
                break;
            }
        }
        if (!empty($categoryId)) {
            $query .= " AND `category_id` = {$categoryId}";
        }
        
        if (!empty($subCategoryId)) {
            $query .= " AND `sub_category_id` = {$subCategoryId}";
        }
        if (!empty($blockedUserIdsString)) {
            $query .= " AND `user_id` NOT IN ({$blockedUserIdsString})";
        }
        $query .= " AND `deleted_at` IS NULL ORDER BY `start_time` ASC LIMIT {$limit} OFFSET {$offset}";

        $allUpcomingSpaces = DB::select($query);
        if (!empty($allUpcomingSpaces)) {
            $spaceModel = new Space;
            $spaces = $spaceModel->getSpaces($allUpcomingSpaces, $loggedInUserId);
            return response()->json(['code' => 200, 'message' => 'Spaces fetched successfully', 'data' => $spaces], 200);
        }
        return response()->json(['code' => 200, 'message' => 'Spaces not found','data'=>$allUpcomingSpaces], 200);
    }
    public function searchSpaces(Request $request)
    {
        $searchTitle    = $request->searchTitle; 
        $categoryId     = $request->category_id;
        $subCategoryId  = $request->sub_category_id;
        
        $user = $request->user();
        $loggedInUserId = $user->id;
        $limit = $request->limit ?? 10;
        $offset = $request->offset ?? 0;
        $followedUserIds = Followers::where('from_user_id', $loggedInUserId)->pluck('to_user_id')->toArray();
        
        $query = "SELECT * FROM `spaces` WHERE ";
        $blockedUserIds = BlockUser::where('from_user_id', $loggedInUserId)->pluck('to_user_id')->toArray();
        $blockedUserIdsString = !empty($blockedUserIds) ? implode(',', $blockedUserIds) : '';

        $followedUserIdsString = !empty($followedUserIds) ? implode(',', $followedUserIds) : '';
        
        $query .= " (
            `privacy` = 1";
    
        if (!empty($followedUserIdsString)) {
            $query .= " OR (`privacy` = 2 AND `user_id` IN ($followedUserIdsString))";
        }
    
        $query .= " OR (`privacy` = 3 AND `id` IN (SELECT space_id FROM space_check_ins WHERE is_private_space=1 AND user_id = {$loggedInUserId})) ";
    
        $query .= " ) AND `status` != 'completed' AND `user_id` != {$loggedInUserId}";
        
        
        
        // Add search filter based on title
        if (!empty($searchTitle)) {
            $searchTitle = '%' . $searchTitle . '%';
            $query .= " AND `title` LIKE '{$searchTitle}'";
        }
        
        // Add time filter only if the type is 'upcoming'
        
        if (!empty($categoryId)) {
            $query .= " AND `category_id` = {$categoryId}";
        }
        
        if (!empty($subCategoryId)) {
            $query .= " AND `sub_category_id` = {$subCategoryId}";
        }
        if (!empty($blockedUserIdsString)) {
            $query .= " AND `user_id` NOT IN ({$blockedUserIdsString})";
        }
        $query .= " AND `deleted_at` IS NULL ORDER BY `start_time` ASC LIMIT {$limit} OFFSET {$offset}";
       
        $allUpcomingSpaces = DB::select($query);
        if (!empty($allUpcomingSpaces)) {
            $spaceModel = new Space;
            $spaces = $spaceModel->getSpaces($allUpcomingSpaces, $loggedInUserId);
            return response()->json(['code' => 200, 'message' => 'Spaces fetched successfully', 'data' => $spaces], 200);
        }
        return response()->json(['code' => 200, 'message' => 'Spaces not found','data'=>$allUpcomingSpaces], 200);
    }
    
    public function joinSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $user  = $request->user();
        $space = Space::find($space_id);
        if (empty($space)) {
            return response()->json([
                'code' => 400,
                'error_message' => 'space not found',
            ]);
        }
        
        if ($space->user_id == $user->id) {
            return response()->json([
                'code' => 400,
                'error_message' => 'Cannot join your own space',
            ]);
        }
        $checkOldJoin = SpaceMember::where(['user_id'=>$user->id,'space_id'=>$space_id])->first();
        if(!empty($checkOldJoin) && $space->status=='active')
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'You already joined space',
            ]);
        }

        $userwallet = getUserWallet($user->id);
        $transactionModel = New Transaction();
        $spaceMemberModel = New SpaceMember();

        if($space->status=='upcoming')
        {
            if($space->privacy!=3)
            {
                $checkoldJoin = SpaceCheckIn::where(['space_id'=>$space_id,'user_id'=>$user->id])->first();
                if(!empty($checkoldJoin))
                {
                    return response()->json([
                        'code' => 400,
                        'error_message' => 'You already join space is not started wait for admin to start ',
                    ]);
                }
                if($space->is_paid==1)
                {
                    if($transactionModel->enoughAmount($space->amount,$userwallet))
                    {
                        $transactionModel->transactionData($space->amount,$space->user_id,$user->id,$space->id);
                        SpaceCheckIn::create(['space_id'=>$space_id,'user_id'=>$user->id,'payment_status'=>1]);

                        return response()->json([
                            'code' => 200,
                            'message' => 'Space is joined wait for admin to start  ',
                        ]);
                    } 
                    else{
                        return response()->json([
                            'code' => 400,
                            'error_message' => 'Insufficent balance ',
                        ]);
                    }
                }
                else
                {
                    $checkoldJoin = SpaceCheckIn::create(['space_id'=>$space_id,'user_id'=>$user->id]);
                    return response()->json([
                        'code' => 200,
                        'message' => 'Space is joined wait for admin to start  ',
                    ]);
                }
            }else
            {
                return response()->json([
                    'code' => 400,
                    'error_message' => 'Can not join private space  ',
                ]);
            }
        }
        elseif($space->status=='active')
        {
            if($space->privacy!=3)
            {
                
                if($space->is_paid==1)
                {
                   
                    $checkoldJoin = SpaceCheckIn::where(['space_id'=>$space_id,'user_id'=>$user->id,'payment_status'=>1])->first();
                    if(!empty($checkoldJoin))
                    {
                        $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,$checkoldJoin->is_cohost);
                        $space->increment('members');
                        return response()->json([
                            'code' => 200,
                            'message' => 'Space joined successfully',
                        ]);
                    }
                    else
                    {
                        if($transactionModel->enoughAmount($space->amount,$userwallet))
                        {
                            $transactionModel->transactionData($space->amount,$space->user_id,$user->id,$space->id);
                            $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,0);
                            $space->increment('members');
                            return response()->json([
                                'code' => 200,
                                'message' => 'Space joined successfully',
                            ]);
                        } 
                        else
                        {
                            return response()->json([
                                'code' => 400,
                                'error_message' => 'Insufficent Balance',
                            ]);
                        }
                    }
                }
                else
                {
                 
                    $checkoldJoin = SpaceCheckIn::where(['space_id'=>$space_id,'user_id'=>$user->id])->first();
                    if(!empty($checkoldJoin))
                    {
                        $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,$checkoldJoin->is_cohost);
                        $space->increment('members');
                        return response()->json([
                            'code' => 200,
                            'message' => 'Space joined successfully',
                        ]);
                    }
                    else
                    {
                        $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,0);
                        $space->increment('members');
                        return response()->json([
                            'code' => 200,
                            'message' => 'Space joined successfully',
                        ]);
                    }
                }

            }
            else
            {
                $checkoldJoin = SpaceCheckIn::where(['space_id'=>$space_id,'user_id'=>$user->id,'is_private_space'=>1])->first();
                if(!empty($checkoldJoin))
                {
                    if($space->is_paid==1)
                    {
                        if($checkoldJoin->payment_status==0)
                        {
                            if($transactionModel->enoughAmount($space->amount,$userwallet) )
                            {
                                $transactionModel->transactionData($space->amount,$space->user_id,$user->id,$space->id);
                                $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,$checkoldJoin->is_cohost);
                                $space->increment('members');
                                return response()->json([
                                    'code' => 200,
                                    'message' => 'Space joined successfully  ',
                                ]);
                            } 
                            return response()->json([
                                'code' => 400,
                                'error_message' => 'Insufficent Balance',
                            ]);
                        }
                        else
                        {
                            $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,$checkoldJoin->is_cohost);
                            $space->increment('members');
                            return response()->json([
                                'code' => 200,
                                'message' => 'Space joined successfully  ',
                            ]);
                        }
                       
                    }
                    $spaceMemberModel->createSpaceMember($space->privacy,$user->id,$space_id,$checkoldJoin->is_cohost);
                    $space->increment('members');
                    return response()->json([
                        'code' => 200,
                        'message' => 'Space joined successfully ',
                    ]);
                }
                else
                {
                    return response()->json([
                        'code' => 400,
                        'error_message' => 'Can not join  private space',
                    ]);
                }

            }
           
        }
        else{
            return response()->json([
                'code' => 400,
                'error_message' => 'Space is not live',
            ]);
        }
       
       
       
    }
    public function sendNotification($from_user_id,$to_user_id,$text,$notification_type,$space_id=0,$gift_id=0)
    {
        $from_user = User::select(['username','profile_image'])->where('id',$from_user_id)->first();
        $to_user_id = User::select(['username','profile_image'])->where('id',$to_user_id)->first();
        
    }
    public function pendingJoinRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $user  = $request->user();
        $space = Space::find($space_id);
        if (empty($space)) {
            return response()->json([
                'code' => 400,
                'error_message' => 'space not found',
            ]);
        }
        if ($space->user_id != $user->id) {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        $users = [];
        $spacemembers = SpaceMember::where(['space_id'=>$space_id,'join_status'=>2])->get();
        if(!empty($spacemembers))
        {
            $userModel = New User;
            $user = [];
            foreach($spacemembers as $member)
            {
                $user = $userModel->usershortInfo($member->user_id);
                $user['request_id'] = $member->id;
                $users[] = $user;
            }
            return response()->json([
                'code' => 200,
                'message' => 'Pending Request fetch  successfully',
                'data'=>$users
            ]);
        }
    }
    public function approveSpaceJoinRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'request_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $user = $request->user();
        $request_id= $request->request_id;
       
        $joinRequest = SpaceMember::where(['id'=>$request_id,'join_status'=>2])->first();
       
        if(empty($joinRequest))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Join request not found',
            ]);
        }
       
        $space = Space::find($joinRequest->space_id);
        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if($user->id != $space->user_id)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        $joinRequest->join_status = 1;
        $joinRequest->update();
        $space->increment('members');
        return response()->json([
            'code' => 200,
            'messsage' => 'Request approved successfully',
        ]);
    }
    public function removeMember(Request $request){
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'user_id'=>'required',
            
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $user_id = $request->user_id;
        $space = Space::find($space_id);
        $loggedInUser = $request->user();

        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if($loggedInUser->id != $space->user_id)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        $userrecord = SpaceMember::where(['user_id'=>$user_id,'space_id'=>$space_id])->first();
        if(empty($userrecord))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'User not found',
            ]);
        }
        $userrecord->delete();
        $space->decrement('members');
        return response()->json([
            'code' => 200,
            'message' => 'The user is removed successfully',
        ]);

    }
    public function addCoHost(Request $request){
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'user_ids'=>'required',
            
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $userIdString = $request->user_ids;
        $userIdArray = explode(",", $userIdString);
        $loggedInUser = $request->user();
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if($loggedInUser->id != $space->user_id)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        if(count($userIdArray)>0)
        {
            foreach($userIdArray as $user_id)
            {
                $user = User::find($user_id);
                $userrecord = SpaceMember::where(['user_id'=>$user_id,'space_id'=>$space_id,'is_cohost'=>0,'join_status'=>1])->first();
                if(!empty($userrecord) && !empty($user) )
                {
                    $userrecord->is_cohost= 1;
                    $userrecord->update();
                }
            }
            return response()->json([
                'code' => 200,
                'message' => 'Cohost added successfully',
            ]);
    
        }
        return response()->json([
            'code' => 400,
            'message' => 'Please select listener to make the cohost',
        ]);

    }
    public function removeCoHost(Request $request){
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'user_ids'=>'required',
            
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $userIdString = $request->user_ids;
        $space = Space::find($space_id);
        $userIdArray = explode(",", $userIdString);
        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if((in_array($space->user_id, $userIdArray)))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        foreach($userIdArray as $user_id)
        {
            $user =  User::find($user_id);
            $userrecord = SpaceMember::where(['user_id'=>$user_id,'space_id'=>$space_id,'is_cohost'=>1])->first();
            if(!empty($user) && !empty($userrecord))
            {
                $userrecord->is_cohost= 0;
                $userrecord->update();
            }
            return response()->json([
                'code' => 200,
                'message' => 'Cohost removed successfully',
            ]);
        }
        return response()->json([
            'code' => 200,
            'message' => 'Please select speaker to remove ',
        ]);
        
    }
    public function activeSpace(Request $request){
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',            
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $loggedInUser = $request->user();
        $user_id = $loggedInUser->id;
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if($user_id != $space->user_id)
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Un Authenticated',
            ]);
        }
        Space::where(['user_id'=>$user_id,'status'=>'active'])->update(['status'=>'completed']);
        $space->status = 'active';
        $space->update();
        $spaceUsers = SpaceCheckIn::where('space_id',$space_id)->get();
        $devices = [];
        if(!empty($spaceUsers))
        {
            foreach($spaceUsers as $spaceuser )
            {
                $spaceuserdata = User::select(['device_id','pause_notification'])->where('id',$spaceuser->user_id)->first();
                
                if(!empty($spaceuserdata) && $spaceuserdata->pause_notification==0)
                {
                    Notification::create([
                        'from_user_id'=>$space->user_id,
                        'to_user_id'=>$spaceuser->user_id,
                        'space_id'=>$space_id,
                        'notification_type'=>'active_space',
                        'text'=>'Space is active please join the space'
                    ]);
                    $devices[] =$spaceuserdata->device_id;  
                }
                $username = $loggedInUser->username;
                $profile_image = getMedia($loggedInUser->profile_image,'user_profile');
                }
                if(!empty($devices))
                {
                    sendPushNotificaiton($space->title.'is live now. Click to join the space',$devices,$profile_image,'','Active space');
                }
            return response()->json([
                'code' => 200,
                'message' => 'Space is active ',
            ]);
        }
        return response()->json([
            'code' => 200,
            'message' => 'Space is activated',
        ]);
    }
    public function sendGift(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'user_id'=>'required',
            'gift_id'=>'required',    
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $gift_id = $request->input('gift_id');
        $user_id = $request->input('user_id');
        $loggedInUser = $request->user()->id;

        // Check if the user is trying to send a gift to themselves
        if ($user_id == $loggedInUser) {
            return response()->json([
                'code' => 400,
                'error_message' => 'You cannot send the gift to yourself',
            ], 200);
        }
        $gift = Gift::find($gift_id);
        $user = User::find($user_id);

        // Check if the gift exists
        if (!$gift) {
            return response()->json([
                'code' => 400,
                'error_message' => 'Gift not found',
            ], 200);
        }
        $user_balance = getUserWallet($loggedInUser);

        if ($gift->price > $user_balance) {
            return response()->json([
                'code' => 400,
                'error_message' => 'Insufficient balance',
            ], 200);
        }
        Transaction::create([
            'user_id' => $user_id,
            'flag' => 'C',
            'description' => 4,
            'gift_id' => $gift_id,
            'amount' => $gift->price,
        ]);

        Transaction::create([
            'user_id' => $loggedInUser,
            'flag' => 'D',
            'description' => 4,
            'gift_id' => $gift_id,
            'amount' => $gift->price,
        ]);
        Notification::create([
            'from_user_id'=>$loggedInUser,
            'to_user_id'=>$user_id,
            'gift_id'=>$gift_id,
            'notification_type'=>'deposit_amount',
            'text'=>'sent you a gift'
        ]);
        Notification::create([
            'from_user_id'=>$user_id,
            'to_user_id'=>$loggedInUser,
            'gift_id'=>$gift_id,
            'notification_type'=>'deductt_amount',
            'text'=>'deduct amount due to sending gift'
        ]);
        return response()->json([
            'code' => 200,
            'message' => 'Gift successfully sent.',
        ], 200);
    }
    public function updateSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $space = Space::find($space_id);
        if(empty($space)){
            return response()->json(['code'=>400,'error_message'=>'Space not found'], 200);
        }
        $inputData = $request->all();
        $user = $request->user();
        $updateData = [];
        foreach ($inputData as $fieldName => $fieldValue) {
            if($fieldName!='space_id' && $fieldName!='cohost_ids' )
            {
                $updateData[$fieldName] = $fieldValue;
            }
            if($fieldName=='start_time')
            {
                $updateData[$fieldName] = convertTimeToUTC($fieldValue);
            }
        }        
        Space::where('id', $space_id)->update($updateData);
        $cohosts = $request->cohost_ids;
        
        if(!empty($cohosts))
        {
            $cohostArray = explode(',', $cohosts);
            
            SpaceCheckIn::where('space_id',$space_id)->whereNotIn('user_id',$cohostArray)->delete();
            foreach ($cohostArray as $CohostId) {
                SpaceCheckIn::updateOrCreate(
                    ['space_id' => $space_id, 'user_id' => $CohostId,'is_cohost'=>1], 
                );
            } 
        }
        return response()->json([
            'code' => 200,
            'message' => 'Space is updated',
        ]);
    }
    public function getSpaceMembers(Request $request)
    { 
        $validator = Validator::make($request->all(), [
        'space_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $userModel = New User;
        $speakers = [];
        $linters = [];
        $limit = $request->limit??10;
        $offset = $request->offset??0;
        
        $space = Space::find($space_id);
        if(empty($space)){
            return response()->json(['code'=>400,'error_message'=>'space not found '], 200);
        }
        if($space->status=='upcoming'){
            return response()->json(['code'=>400,'error_message'=>'space is not active'], 200);
        }
        $speakers  = SpaceMember::where('space_id', $space_id)
                    ->where(function($query) {
                        $query->where('is_host', 1)
                              ->orWhere('is_cohost', 1);
                    })
                    ->limit($limit)
                    ->offset($offset)
            ->get();
        if(!empty($speakers) && count($speakers)>0)
        {
            foreach($speakers as $speaker)
            {
                $userdata = $userModel->usershortInfo($speaker->user_id);
                $userdata['is_host'] = $speaker->is_host;
                $userdata['is_cohost'] = $speaker->is_cohost;
                $allMembers['speakers'][] = $userdata;
            }
        }
        
        $allMembers['listeners'] =[];
        $listners  = SpaceMember::where(['space_id'=>$space_id,'is_host'=>0,'is_cohost'=>0])->limit($limit)->offset($offset)->get();
        $allMembers['total_members_count'] =  count($listners) + count($speakers);
        if(!empty($listners) )
        {
            foreach($listners as $listner)
            {
                $userdata = $userModel->usershortInfo($listner->user_id);
                $userdata['is_host'] = 0;
                $userdata['is_cohost'] = 0;
                $allMembers['listeners'][] = $userdata;
            }
             
        }
        $allMembers['followusers'] = [];
      
        $followusers =  SpaceMember::where('space_id', $space_id)
            ->whereIn('user_id', function($query) {
                $query->select('users.id')
                    ->from('users')
                    ->join('followers', 'users.id', '=', 'followers.to_user_id')
                    ->where('followers.from_user_id', auth()->id());
            })
            ->where('user_id', '!=', auth()->id())
            ->get();
        
        if(!empty($followusers) )
        {
            foreach($followusers as $followuser)
            {
                $followuserdata = $userModel->usershortInfo($followuser->user_id);
                $followuserdata['is_host'] = $followuser->is_host;
                $followuserdata['is_cohost'] = $followuser->is_cohost;
                $allMembers['followusers'][] =  $followuserdata;
            }
        }
        
        return response()->json([
            'code' => 200,
            'message' => 'Space members fetch successfully',
            'data'=>$allMembers,
        ]);
    }
    public function generateAgoraToken(Request $request)
    {
        $appID = get_setting('agora_app_id');
        $appCertificate = get_setting('agora_certificate');
        $channelName = $request->user()->email;
        $role = RtcTokenBuilder::RolePublisher;
        $expireTimeInSeconds = 7200;
        $currentTimestamp = now()->getTimestamp();
        $privilegeExpiredTs = $currentTimestamp + $expireTimeInSeconds;
        $token = RtcTokenBuilder::buildTokenWithUid($appID, $appCertificate, $channelName, 0, $role, $privilegeExpiredTs);
        return json_encode(['status' => 200, 'message' => "token generated successfully", 'token' => $token]);
    }
    public function leaveSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
        'space_id'=>'required',
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $loggedInUserId = $request->user()->id;
        $space =Space::find($space_id);
        if(empty($space))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space not found',
            ]);
        }
        if($space->status!='active')
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'Space is not live',
            ]);
        }
        if($space->user_id==$loggedInUserId)
        {
            $space->status = 'completed';
            $space->update();
            $user  = User::find($space->user_id);
            if($user->status==2)
            {
                $user->tokens()->delete();
            }
            return response()->json([
                'code' => 200,
                'message' => 'Space left successfully',
            ]);
        }
        $spaceMember = SpaceMember::where(['space_id'=>$space_id,'user_id'=>$loggedInUserId])->first();
        if(empty($spaceMember))
        {
            return response()->json([
                'code' => 400,
                'error_message' => 'User is not in this space',
            ]);
        }
        $spaceMember->delete();
        $space->decrement('members');
        return response()->json([
            'code' => 200,
            'message' => 'Space left successfully',
        ]);
    }
    public function addSpaceMembers(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'member_ids'=>'required'
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $space_id = $request->space_id;
        $memberIdStrubg = $request->member_ids;
        $loggedInUser  = $request->user();
        $loggedInuserId  = $loggedInUser->id;
        
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json(['code'=>400,'error_message'=>'Space not found'], 200);
        }
        $memberIdArray = explode(',', $memberIdStrubg);
        $memberdeviceId = [];
        if(count($memberIdArray)>0)
        {
            foreach($memberIdArray as $memberId)
            {
                $memberdata = User::select(['device_id','notify_add_member'])->where('id',$memberId)->first();
                if(empty($memberdata))
                {
                    return response()->json(['code'=>400,'error_message'=>'User not found'], 200);
                }
                SpaceCheckIn::create([
                    'space_id'=>$space_id,
                    'user_id'=>$memberId,
                    'is_private_space'=>1,
                    'payment_status'=>0   
                ]);
                $text = "invited you to join space ";
                if($memberdata->id!=$loggedInuserId && $memberdata->notify_add_member==1)
                {
                    Notification::create([
                        'from_user_id' =>$loggedInuserId,
                        'to_user_id' =>$memberId,
                        'space_id'=>$space_id,
                        'text'=>$text,
                        'notification_type'=>'add_member'
                    ]);
                    $username = $loggedInUser->username;
                    $profile_image = getMedia($loggedInUser->profile_image);
                    if(!empty($memberdata->device_id))
                    {
                        $memberdeviceId[] = $memberdata->device_id;
                        // sendPushNotificaiton($username,$memberdata->device_id,$profile_image,$text);
                    }    
                }
            }
        }
        if(count($memberdeviceId)>0)
        {
            $memberdeviceId[] = $memberdata->device_id;
            sendPushNotificaiton($username,$memberdeviceId,$profile_image,$text,'Space Invitation');
        }    
        return response()->json(['code'=>200,'message'=>'Members are invited'], 200);
    }
    public function inviteMember(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'user_id'=>'required',
        ]);

        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $loggedInUser = $request->user();
        $space_id = $request->space_id;
        $user_id = $request->user_id;
        $space   = Space::find($space_id);
        $user   = User::find($user_id);
        if(empty($space))
        {
            return response()->json(['code'=>400,'error_message'=>'Space not found'], 200);
        }
        if(empty($user))
        {
            return response()->json(['code'=>400,'error_message'=>'User not found'], 200);
        }
        if(empty($space->user_id !=$loggedInUser->id ))
        {
            return response()->json(['code'=>400,'error_message'=>'User not found'], 200);
        }
        $spacecheckin = New SpaceCheckIn();
        $spacecheckin->user_id = $user_id;
        $spacecheckin->space_id = $space_id;
        $spacecheckin->is_private_space  = ($space->privacy==3)?1:0;
        $spacecheckin->payment_status = 0;
        $spacecheckin->save();
        if($user->pause_notification==0)
        {
            $text = 'Invited you to join space';
            Notification::create([
                'from_user_id' =>$loggedInUser->id,
                'to_user_id' =>$user_id,
                'space_id'=>$space_id,
                'text'=>$text,
                'notification_type'=>'checked_in_space'
            ]);
            $username = $loggedInUser->username;
            $profile_image = getMedia($loggedInUser->profile_image);
            sendPushNotificaiton($username,$user->device_id,$profile_image,$text,'Space invitation');  
        }
        return response()->json(['code'=>200,'message'=>' Member Invited successfully'], 200);


    }
    public function checkandreversetransaction()
    {
        $currentTimePlus10Minutes = date('Y-m-d H:i:s', strtotime('-10 minutes'));

        $sql = "SELECT * 
                FROM spaces 
                WHERE start_time < '{$currentTimePlus10Minutes}' AND status='upcoming'";
        
        $spaces = DB::select($sql);
        if(!empty($spaces))
        {
            foreach($spaces  as $space )
            {
               if($space->is_paid==1)
                {
                    $spaceusers = SpaceMember::where(['space_id'=>$space->id])->where('user_id','!=',$space->user_id)->pluck('user_id')->toArray();
                    if(!empty($spaceusers))
                    {
                        foreach($spaceusers as $user)
                        {
                            $transactions = Transaction::where(['user_id'=>$user,'space_id'=>$space->id,'flag'=>'D','gift_id'=>0])->first();
                            if(!empty($transactions))
                            {
                                $this->reverseTransaction($transactions->amount,$user,$space->user_id);
                            }
                        }
                    }
                }
                Space::where('id',$space->id)->update(['status'=>'completed']);
            }
        }

    }
    public function reverseTransaction($amount,$touser,$fromuser)
    {
        $creditTransaction = New Transaction;
        $creditTransaction->user_id = $fromuser;
        $creditTransaction->flag = 'D';
        $creditTransaction->description = 1;
        $creditTransaction->amount = $amount;
        $creditTransaction->save();
        $creditTransaction = New Transaction;
        $creditTransaction->user_id = $touser;
        $creditTransaction->flag = 'C';
        $creditTransaction->description = 1;
        $creditTransaction->amount = $amount;
        $creditTransaction->save();
    }
    public function sendPushNotification(Request $request)
    {
        $app_id = get_setting('one_signal_app_id');
        $app_key = get_setting('one_signal_app_key');

        $device = ['49dd0f98-9d40-4c8d-8e5c-53dac17a0bbb'];

        $final_request_data = array(
            'app_id' => $app_id,
            'include_player_ids' => $device,
            'contents' => array(
                'en' => 'testing Notification 1'
            ),
            'headings' => array(
                'en' => 'Saifrehman'
            ),
            'android_led_color' => 'FF0000FF',
            'priority' => 10,
            'large_icon' => 'https://club92.socioon.com/uploads/photos/user_profile/2024/07/16.jpg'
        );

        $fields = json_encode($final_request_data);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, "https://onesignal.com/api/v1/notifications");
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json; charset=utf-8',
            'Authorization: Basic ' . $app_key
        ));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        $response = curl_exec($ch);

        if (curl_errno($ch)) {
            // Error handling
            echo 'Error:' . curl_error($ch);
        } else {
            $responseData = json_decode($response, true);
            if ($responseData === null) {
                echo '<div style="color: red;">Invalid JSON response from OneSignal API</div>';
            } else {
                // Print response for debugging
                print_r($responseData);
            }
        }

        curl_close($ch);
    }
    public function reportSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
            'reason'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
            
        }
        $space_id = $request->space_id;
        $loggedInUser = $request->user();
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json(['code'=>400,'error_message'=>'space not found'], 200);
            
        }
        if($space->user_id==$loggedInUser->id)
        {
            return response()->json(['code'=>400,'error_message'=>"Can't report your own space"], 200);
        }
        if($space->status!='active')
        {
            return response()->json(['code'=>400,'error_message'=>"You can  report only live space"], 200);
        }
        $checkexistreport = ReportSpace::where(['user_id'=>$loggedInUser->id,'space_id'=>$space_id])->first();
        if(!empty($checkexistreport))
        {
            return response()->json(['code'=>400,'error_message'=>"You already reported this space"], 200);
        }
        ReportSpace::create(['user_id'=>$loggedInUser->id,'space_id'=>$space_id,'reason'=>$request->reason]);
        return response()->json(['code'=>200,'message'=>"space reported successfully"], 200);
    } 
    public function getGfits(Request $request)
    {
        $limit = $request->limit??10;
        $offset = $request->offset??0;
        $gifts = Gift::limit($limit)->offset($offset)->get();
        if(!empty($gifts))
        {
            foreach($gifts as $gift)
            {
                $gift['image'] = getMedia($gift->image);
            }
            return response()->json(['code'=>200,'message'=>"Gift fetch successfully",'data'=>$gifts], 200);
            
        }
        return response()->json(['code'=>200,'message'=>"Gift not found",'data'=>$gifts], 200);

    }
    public function spaceDetails(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $space_id = $request->space_id;
        $space = Space::find($space_id);
        if(empty($space)){
            return response()->json(['code'=>400,'error_message'=>'space not found'], 200);
        }
        // if($space->status=='completed')
        // {
        //     return response()->json(['code'=>400,'error_message'=>'Space is completed'], 200);
            
        // }
        // if($space->status=='reported')
        // {
        //     return response()->json(['code'=>400,'error_message'=>'Space is reported '], 200);
        // }

        $spaceModel = New Space;
        
        $spaceDetail = $spaceModel->getsingleSpace($space,Auth::id());
        return response()->json(['code'=>200,'message'=>'space details fetch successfully','data'=>$spaceDetail], 200);

    }
    public function deleteSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $space_id = $request->space_id;
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json(['code'=>400,'error_message'=>'space not found'], 200);
        }
        if($space->status!='upcoming')
        {
            return response()->json(['code'=>400,'error_message'=>'Can Delete Only Upcoming space '], 200);
        }
        if($space->is_paid==1)
        {
            $space_users = SpaceCheckIn::where(['space_id'=>$space->id,'payment_status'=>1,'is_cohost'=>0])->pluck('user_id')->toArray();
            foreach($space_users as $joiner)
            {
                $transactionModel = New Transaction();
                $transactionModel->reverseTransaction($space,$joiner);
            }
        }    
        SpaceCheckIn::where(['space_id'=>$space->id])->delete();
        Space::where('id',$space_id)->delete();
        return response()->json(['code'=>200,'message'=>'Space deleted successfully'], 200);
    }
    public function  checkSpace(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'space_id'=>'required',
        ]);
        if($validator->fails()){
            return response()->json(['code'=>400,'error_message'=>$validator->errors()->first()], 200);
        }
        $space_id = $request->space_id;
        $space = Space::find($space_id);
        if(empty($space))
        {
            return response()->json(['code'=>400,'error_message'=>'space not found'], 200);
        }
        $users = SpaceMember::join('users', 'users.id', '=', 'space_members.user_id')
            ->where('space_members.space_id', $space_id)
            ->where('users.status', 2)
            ->pluck('space_members.user_id')
        ->toArray();
        $is_reported = 0;
        $user = User::select('status')->where('id',$space->user_id)->first();
        if($space->is_reported==1)
        {
            $is_reported = 1; 
        }
        $data['is_reported'] = $is_reported;
        $data['is_admin_reported'] = ($user->status==2)?1:0;
        $data['users'] = $users;
        return response()->json(['code'=>200,'message'=>'Space checked successfully','data'=>$data], 200);
    }
}