<?php

namespace App\Http\Controllers\Api;

use App\Models\User;
use App\Models\Story;
use App\Models\Followers;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\SeenStory;
use Illuminate\Support\Facades\Validator;

class StoryController extends Controller
{
    public function getStories(Request $request)
    {
       $limit = $request->limit ?? 10;
       $offset = $request->offset ?? 0;
       $loggedInUserId  = $request->user()->id;
       $users =  Followers::where('from_user_id',$loggedInUserId)->pluck('to_user_id')->toArray();
       $usersallstoriesdata = [];
       array_unshift($users, $loggedInUserId);
       if(count($users)>0)
        {
            $userModel  = New User();
            $storyModel = New Story();
            foreach($users as $user)
            {
                $stories  = $storyModel->getStories($user);
                if($stories!=null && count($stories)>0)
                {
                    $userdata['users_info'] = $userModel->usershortInfo($user,$loggedInUserId);
                    $userdata['stories'] = $stories;
                    $usersallstoriesdata[] = $userdata;
                }
            }
            if(count($usersallstoriesdata)>0)
            {
                return response()->json(['code'=>200,'message'=>'Stories fetch successfully','data'=>$usersallstoriesdata], 200);
            }
            
        }
        if(count($usersallstoriesdata)==0)
        {
            return response()->json(['code'=>400,'error_message'=>'Stories not found','data'=>$usersallstoriesdata], 200);
        }


    }
    public function createStory(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'type'=>'required',
            'description'=>'required_if:type,1',
            'media'=>'required_if:type,2',
            'thumbnail'=>'required_if:type,3',
            
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $type = $request->type;
        $story = New Story;
        $loggedInUser = $request->user();
        $story->user_id =$loggedInUser->id;
        $story->type = $type;
        if($type==1)
        {
            $story->description  = $request->description;
        }
        if($type==2)
        {
            $media = $request->file('media');
            if(!empty($media))
            {
                $story->media = storeMedia($media,'story');
            }
        }
        if($type==3)
        {
            $thumbnail = $request->file('thumbnail');
            if(!empty($thumbnail))
            {
                $story->thumbnail = storeMedia($thumbnail,'story_thumbnail');
            }
            $media = $request->file('media');
            if(!empty($media))
            {
                $story->media = storeMedia($media,'story');
            }
        }
  
        $story->save();
        $storydata = $story->getStoryDetails($story->id);
        return response()->json(['code'=>200,'message'=>'Story is created successfully','data'=>$storydata], 200);        
        
    }
    public function deleteStory(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'story_id'=>'required'
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $story_id = $request->story_id;
        $user_id = $request->user()->id;
        $story = Story::find($story_id);
        if(empty($story))
        {
            return response()->json(['code'=>400,'error_message'=>'story not found'], 200);
        }
        if($story->user_id!=$user_id)
        {
            return response()->json(['code'=>400,'error_message'=>'Un Authenticated'], 200);
        }    
        $story->delete();    
        return response()->json(['code'=>200,'message'=>'Story deleted successfully'], 200);
    }
    public function ViewStory(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'story_id'=>'required'
        ]);
        if($validator->fails()){
            $first_error = $validator->errors()->first();
            return response()->json(['code'=>400,'error_message'=>$first_error], 200);
        }
        $story_id = $request->story_id;
        $user_id = $request->user()->id;
        $story = Story::find($story_id);
        if(empty($story))
        {
            return response()->json(['code'=>400,'error_message'=>'story not found'], 200);
        }
        if($story->user_id==$user_id)
        {
            return response()->json(['code'=>400,'error_message'=>'This is your own story'], 200);
        }    
        $viewstory = SeenStory::where(['user_id'=>$user_id,'story_id'=>$story_id])->first();
        if(!empty($viewstory))
        {
            return response()->json(['code'=>400,'error_message'=>'Story already viewed'], 200);
        }
        $story->increment('views_count');    
        SeenStory::create(['user_id'=>$user_id,'story_id'=>$story_id]);
        return response()->json(['code'=>200,'message'=>'Story viewed successfully'], 200);
    }
}
