<?php
namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Session;

class InstallController extends Controller
{
    public function showForm()
    {

        if ($this->envFileExists()) {
            $envContent = File::get(base_path('.env'));
            
            preg_match_all('/(DB_HOST=)(.*)/', $envContent, $matches);
            $currentDbHost = isset($matches[2][0]) ? $matches[2][0] : '';       
            preg_match_all('/(DB_PORT=)(.*)/', $envContent, $matches);
            $currentDbPort = isset($matches[2][0]) ? $matches[2][0] : '';
            preg_match_all('/(DB_DATABASE=)(.*)/', $envContent, $matches);
            $currentDbDatabase = isset($matches[2][0]) ? $matches[2][0] : '';
            preg_match_all('/(DB_USERNAME=)(.*)/', $envContent, $matches);
            $currentDbUsername = isset($matches[2][0]) ? $matches[2][0] : '';
            preg_match_all('/(DB_PASSWORD=)(.*)/', $envContent, $matches);
            $currentDbPassword = isset($matches[2][0]) ? $matches[2][0] : '';
            if ( $currentDbDatabase ==''&& $currentDbUsername =='' && $currentDbPassword =='')
            {
                return view('verifycode');
            }
            else
            {
                return redirect('/')->with('error', 'The application is already installed.');
            }
        }
    }
    public function verify(Request $request)
    {
        $request->validate([
          'verify_email'       =>'required|email',
          'purchasecode'=>'required',
        ]);
        $domain = request()->root();
        $email = $request->verify_email;
        
        $purchase_code = $request->purchasecode;
        $verificationUrl = "https://verify.linkon.social/index.php";

            // Prepare the POST data
            $postData = [
                'purchase_code' => $purchase_code,
                'verify_email' => $email,
                'domain_name' => $domain,
                'product' =>'53620321'
            ];
            
            // Initialize cURL session
            $ch = curl_init();
    
            // Set cURL options
            curl_setopt($ch, CURLOPT_URL, $verificationUrl);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, true); // Set to POST request
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postData)); // Attach the POST data
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Don't verify the peer's SSL certificate (for development or testing purposes)
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);     // Don't verify the host name against the certificate
    
            // Execute the cURL request
            $response = curl_exec($ch);
           
            // Check if the response is false (cURL failed)
            if ($response === false) {
                $error = curl_error($ch);
                echo "cURL Error: " . $error;
            } else {
                // Check if the response is empty
                if (empty($response)) {
                    echo "No response from server.";
                } else {
                    $response = json_decode($response);
                    if($response->status=='success')
                    {
                        $purchasecodefilePath = base_path('purchase_code.txt');
                        if (!file_exists($purchasecodefilePath)) {
                            file_put_contents($purchasecodefilePath, 'WRITE_PURCHASECODE');
                        }
                        $purchasecodefile = file_get_contents($purchasecodefilePath);
                        $changeabletext = str_replace('WRITE_PURCHASECODE', $purchase_code, $purchasecodefile);
                        file_put_contents($purchasecodefilePath, $changeabletext);
                        Session::put('verify',1);
                        return redirect()->route('installApplication');
                    }
                    else
                    {
                        
                        return redirect()->route('install.form')->with('error', $response->message);
                    }
       
                }
            }
            curl_close($ch);
    }
    public function InstallApplication()
    {
        $checkVerification =  Session::get('verify');
        if($checkVerification==1)
        {
            return view('install');
        }
        else
        {
            return redirect()->route('install.form')->with('error', 'Please verify Your Application Purchase code ');

        }
    }

    public function store(Request $request)
    {
        
       
        $data = $request->validate([
            'db_host' => 'required|string',
            'db_port' => 'required|string',
            'db_database' => 'required|string',
            'db_username' => 'required|string',
        ]);

        $data['db_password'] = !empty($request->db_password)? $request->db_password:'';



        // Test the database connection and run the SQL file
        $result = $this->runSqlFile($data);

        if ($result !== true) {
            return redirect()->back()->with('error', $result);
        }

        // Save database credentials to .env
        $this->updateEnvFile($data);

        // Update the .env file to mark the application as installed
        $this->setEnvInstalled();

        return redirect('/')->with('success', 'Application installed successfully.');
    }

    protected function envFileExists()
    {
        return File::exists(base_path('.env'));
    }

    protected function updateEnvFile($data)
    {
        $envPath = base_path('.env');
        $envContent = File::get($envPath);

        $envContent = preg_replace('/DB_HOST=.*/', 'DB_HOST=' . $data['db_host'], $envContent);
        $envContent = preg_replace('/DB_PORT=.*/', 'DB_PORT=' . $data['db_port'], $envContent);
        $envContent = preg_replace('/DB_DATABASE=.*/', 'DB_DATABASE=' . $data['db_database'], $envContent);
        $envContent = preg_replace('/DB_USERNAME=.*/', 'DB_USERNAME=' . $data['db_username'], $envContent);
        $envContent = preg_replace('/DB_PASSWORD=.*/', 'DB_PASSWORD=' . $data['db_password'], $envContent);

        File::put($envPath, $envContent);
    }

    protected function runSqlFile($data)
    {
        $sqlPath = base_path('database/database.sql');
        $sql = File::get($sqlPath);

        try {
            $db = new \mysqli($data['db_host'], $data['db_username'], $data['db_password'], $data['db_database'], $data['db_port']);

            if ($db->connect_error) {
                return "Connection failed: " . $db->connect_error;
            }

            $multiQuery = $db->multi_query($sql);

            while ($db->more_results()) {
                $db->next_result();
            }

            $db->close();
        } catch (\Exception $e) {
            return "Database connection error: " . $e->getMessage();
        }

        return true;
    }

    protected function setEnvInstalled()
    {
        $envPath = base_path('.env');
        $envContent = File::get($envPath);
        $envContent = preg_replace('/PRE_INSTALLED=.*/', 'PRE_INSTALLED=true', $envContent);

        if (!str_contains($envContent, 'PRE_INSTALLED=')) {
            $envContent .= PHP_EOL . 'PRE_INSTALLED=true';
        }

        File::put($envPath, $envContent);
    }
    public function verifyPurchasCode(Request $request)
    {
        return view('verifypurchase');
    }
    public function verifyPurchaseCodeSubmit(Request $request)
    {
        $request->validate([
            'verify_email'       =>'required|email',
            'purchasecode'=>'required',
        ]);
        $domain = request()->root();
        $email = $request->verify_email;
        $purchase_code = $request->purchasecode;
        $verificationUrl = "https://verify.linkon.social/index.php";

            // Prepare the POST data
            $postData = [
                'purchase_code' => $purchase_code,
                'verify_email' => $email,
                'domain_name' => $domain,
                'product' =>'53620321'
            ];
            
            // Initialize cURL session
            $ch = curl_init();
    
            // Set cURL options
            curl_setopt($ch, CURLOPT_URL, $verificationUrl);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POST, true); // Set to POST request
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postData)); // Attach the POST data
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Don't verify the peer's SSL certificate (for development or testing purposes)
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);     // Don't verify the host name against the certificate
    
            // Execute the cURL request
            $response = curl_exec($ch);
            
            // Check if the response is false (cURL failed)
            if ($response === false) {
                $error = curl_error($ch);
                echo "cURL Error: " . $error;
            } else {
                // Check if the response is empty
                if (empty($response)) {
                    echo "No response from server.";
                } else {
                    $response = json_decode($response);
                    if($response->status=='success')
                    {
                        $purchasecodefilePath = base_path('purchase_code.txt');
                        if (!file_exists($purchasecodefilePath)) {
                            file_put_contents($purchasecodefilePath, 'WRITE_PURCHASECODE');
                        }
                        $purchasecodefile = file_get_contents($purchasecodefilePath);
                        $changeabletext = str_replace('WRITE_PURCHASECODE', $purchase_code, $purchasecodefile);
                        file_put_contents($purchasecodefilePath, $changeabletext);
                        Session::put('verify',1);
                        return redirect()->route('installApplication');
                    }
                    else
                    {
                        return redirect()->route('install.form')->with('error', $response->message);
                    }
        
                }
            }
            curl_close($ch);
     
    }

}
