<?php // app/Services/PayFastService.php

namespace App\Services;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

class PayFastService
{
    protected $client;
    protected $merchantId;
    protected $merchantKey;
    protected $passphrase;
    protected $testMode;

    public function __construct()
    {
        $this->client = new Client();
        $this->merchantId = env('PAYFAST_MERCHANT_ID');
        $this->merchantKey = env('PAYFAST_MERCHANT_KEY');
        $this->passphrase = env('PAYFAST_PASSPHRASE');
        $this->testMode = env('PAYFAST_TEST_MODE', false);
    }

    public function getTransactionDetails($transactionId)
    {
        $url = $this->testMode ? 'https://sandbox.payfast.co.za/eng/query/fetch' : 'https://www.payfast.co.za/eng/query/fetch';
        $params = [
            'merchant_id' => $this->merchantId,
            'merchant_key' => $this->merchantKey,
            'transaction_id' => $transactionId,
        ];

        // Add passphrase if not empty
        if (!empty($this->passphrase)) {
            $params['passphrase'] = $this->passphrase;
        }

        try {
            $response = $this->client->request('GET', $url, [
                'query' => $params
            ]);
            return json_decode($response->getBody()->getContents(), true);
        } catch (RequestException $e) {
            return [
                'error' => true,
                'message' => $e->getMessage(),
            ];
        }
    }
}
