<?php


use Illuminate\Support\Facades\Route;

use App\Http\Controllers\Api\AuthController;
use App\Http\Controllers\Api\PostController;
use App\Http\Controllers\Api\SpaceController;
use App\Http\Controllers\Api\StoryController;
use App\Http\Controllers\Api\PayPalController;
use App\Http\Controllers\Api\ApiUserController;
use App\Http\Controllers\Api\BackendController;
use App\Http\Controllers\Api\PaystackController;
use App\Http\Controllers\Api\ApiDepositController;
use App\Http\Controllers\Api\FlutterwaveController;
use App\Http\Controllers\Api\TransactionController;
use App\Http\Controllers\Api\NotificationController;
use App\Http\Controllers\Api\ApiWithdrawRequestController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

// Route::post('user/register', [AuthController::class,'register']);
// Route::post('user/login', [AuthController::class,'login']);

Route::post('reset-password',[AuthController::class,'resetPassword']);
Route::post('reset-password-verfied',[AuthController::class,'reset_password_verified']);

// Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
//     return $request->user();
// });
// Route::controller(RegisterController::class)->group(function(){
//     Route::post('user/register', 'register');
//     Route::post('user/login', 'login');
     
// });
Route::post('check-username',[AuthController::class,'checkUsername']);
Route::post('check-email',[AuthController::class,'checkExistEmail']);

Route::post('register',[AuthController::class,'register']);
Route::post('login',[AuthController::class,'login']);
Route::post('verify-code',[AuthController::class,'verifyCode']);
Route::post('send-otp',[AuthController::class,'sendOTP']);
Route::get('get-site-setting',[BackendController::class,'getSiteSetting']);
Route::get('term-conditions', [BackendController::class, 'termAndCondition']);
Route::get('privacy-policy', [BackendController::class, 'privcyPolicy']);
Route::get('cookie-policy', [BackendController::class, 'cookiePolicy']);
Route::get('about', [BackendController::class, 'About']);
Route::post('get-space-categories',[SpaceController::class,'getCategories']);

Route::middleware('auth:sanctum')->group( function () {
    Route::post('get-user-data',[ApiUserController::class,'getuserdata']);
    Route::post('update-profile',[ApiUserController::class,'updateProfile']);
    Route::post('search-users',[ApiUserController::class,'searchUser']);
    Route::post('follow-user',[ApiUserController::class,'followUnFollowUser']);
    Route::post('logout',[AuthController::class,'logout']);
    Route::post('block-user',[ApiUserController::class,'blockUser']);
    Route::post('report-user',[ApiUserController::class,'reportUser']);
    Route::post('block-list',[ApiUserController::class,'blockList']);

    Route::post('get-faqs',[BackendController::class,'getFaqs']);
    Route::post('update-interest',[ApiUserController::class,'updateInterest']);
    Route::post('set-speakers',[ApiUserController::class,'updateSpeakers']);
    
    Route::post('get-interest',[ApiUserController::class,'getInterest']);
    Route::post('get-speakers',[ApiUserController::class,'getSpeakers']);
    Route::post('get-user-wallet',[ApiUserController::class,'getUserWallet']);

    
    // Spaces
    Route::post('create-space',[SpaceController::class,'createSpace']);
    Route::post('get-spaces',[SpaceController::class,'getSpaces']);
    Route::post('search-spaces',[SpaceController::class,'searchSpaces']);
    Route::post('join-space',[SpaceController::class,'joinSpace']);
    Route::post('approve-space-join-request',[SpaceController::class,'approveSpaceJoinRequest']);
    Route::post('pending-join-request',[SpaceController::class,'pendingJoinRequest']);
    Route::post('add-cohost',[SpaceController::class,'addCoHost']);
    Route::post('remove-cohost',[SpaceController::class,'removeCoHost']);
    Route::post('remove-member',[SpaceController::class,'removeMember']);
    Route::post('active-space',[SpaceController::class,'activeSpace']);
    Route::post('update-space',[SpaceController::class,'updateSpace']);
    Route::post('generate-agora-token',[SpaceController::class,'generateAgoraToken']);
    Route::post('get-space-members',[SpaceController::class,'getSpaceMembers']);
    Route::post('leave-space',[SpaceController::class,'leaveSpace']);
    Route::post('add-space-lisnters',[SpaceController::class,'addSpaceMembers']);
    Route::post('send-gift', [SpaceController::class, 'sendGift']);
    Route::post('get-gifts', [SpaceController::class, 'getGfits']);
    Route::post('invite-member', [SpaceController::class, 'inviteMember']);
    Route::post('report-space', [SpaceController::class, 'reportSpace']);
    Route::post('space-details', [SpaceController::class, 'spaceDetails']);
    Route::post('delete-space', [SpaceController::class, 'deleteSpace']);
    Route::post('check-space', [SpaceController::class, 'checkSpace']);
    Route::post('get-wallet-balance', [ApiUserController::class, 'getUserWalletData']);
    



    Route::post('deposit-amount',[ApiDepositController::class,'depositAmount']);
    Route::post('withdraw-request', [ApiWithdrawRequestController::class, 'create']);
    Route::post('withdraw-history', [ApiWithdrawRequestController::class, 'withdrawHistory']);
    Route::post('transaction-history', [ApiWithdrawRequestController::class, 'transactionHistory']);

    Route::post('get-unread-notifications', [NotificationController::class, 'getUnreadNotifications']);
    Route::post('get-notifications', [NotificationController::class, 'getAllNotifications']);
    Route::post('mark-as-read', [NotificationController::class, 'markAsRead']);
    Route::post('mark-all-as-read', [NotificationController::class, 'markAllAsRead']);

    Route::post('delete-notification', [NotificationController::class, 'deleteNotitication']);
    Route::post('delete-all-notifications', [NotificationController::class, 'deleteAllNotification']);

    Route::post('transfer-amount', [ApiUserController::class, 'transferFund']);
    
    // 

    Route::post('/paypal/transaction', [PayPalController::class, 'getTransactionDetails']);
    Route::post('/paystack/transaction', [PaystackController::class, 'getTransactionDetails']);
    Route::post('flutter-wave/transaction', [FlutterwaveController::class, 'getTransactionDetails']);
    Route::get('payfast/transaction', [TransactionController::class, 'getTransactionDetails']);


    Route::post('send-notification', [SpaceController::class, 'sendPushNotification']);
    Route::post('testing', [SpaceController::class, 'testingSpace']);

    Route::prefix('post')->group(function(){
        Route::post('add', [PostController::class, 'addPost']);
        Route::post('get-posts', [PostController::class, 'getAllPosts']);
        Route::post('delete-post', [PostController::class, 'deletePost']);
        Route::post('add-comment', [PostController::class, 'AddComment']);
        Route::post('delete-comment', [PostController::class, 'deleteComment']);
        Route::post('share-post', [PostController::class, 'sharePost']);
        Route::post('like-post', [PostController::class, 'likePost']);
        Route::post('get-comments', [PostController::class, 'getPostComments']);
        Route::post('update-post', [PostController::class, 'updatePost']);
        Route::post('report-post', [PostController::class, 'reportPost']);
        Route::post('details', [PostController::class, 'postDetails']);
        
    });
    Route::prefix('story')->group(function(){
        Route::post('add', [StoryController::class, 'createStory']);
        Route::post('get-all', [StoryController::class, 'getStories']);
        Route::post('delete-story', [StoryController::class, 'deleteStory']);
        Route::post('view-story', [StoryController::class, 'viewStory']);
    });
    
});
