<?php

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use App\Http\Middleware\CheckIfInstalled;
use App\Http\Controllers\InstallController;
use App\Http\Controllers\FrontendController;
use App\Http\Controllers\Admin\FAQController;
use App\Http\Controllers\Admin\GiftController;
use App\Http\Controllers\Admin\RoleController;
use App\Http\Controllers\Admin\UserController;
use App\Http\Controllers\Admin\PolicyController;
use App\Http\Controllers\Admin\DepositController;
use App\Http\Controllers\Admin\GatewayController;
use App\Http\Controllers\Admin\SettingController;
use App\Http\Controllers\Admin\CategoryController;
use App\Http\Controllers\Admin\AdminPostController;
use App\Http\Controllers\Admin\DashboardController;
use App\Http\Controllers\Admin\AdminSpaceController;
use App\Http\Controllers\Admin\ManageUserController;
use App\Http\Controllers\Admin\PermissionController;
use App\Http\Controllers\Admin\SubCategoryController;
use App\Http\Controllers\Admin\WithdrawRequestController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/
Route::middleware([CheckIfInstalled::class])->group(function () {
    Route::get('/', function () {
        return redirect()->route('login');
    });
    Auth::routes(['register'=>false]);
});
Route::get('verify-purchase-code',[InstallController::class,'verifyPurchasCode'])->name('getverifyPurchaseCode');
Route::post('verify-purchase-code',[InstallController::class,'verifyPurchaseCodeSubmit'])->name('postverifyPurchaseCode');


Route::get('/install', [InstallController::class, 'showForm'])->name('install.form');
Route::get('/verify', [InstallController::class, 'verify'])->name('verify');

Route::get('/install-application', [InstallController::class, 'InstallApplication'])->name('installApplication');

// Route to handle the form submission and perform the installation
Route::post('/install', [InstallController::class, 'store'])->name('install.store');
Route::post('/verify-code', [InstallController::class, 'verify'])->name('verifyCode');



Route::get('privacy-policy', [FrontendController::class, 'privacyPolicy'])->name('privacyPolicy');
Route::get('term-condition', [FrontendController::class, 'termCondition'])->name('termCondition');
Route::get('cookie-policy', [FrontendController::class, 'cookiePolicy'])->name('cookiePolicy');
Route::get('about-us', [FrontendController::class, 'aboutUs'])->name('aboutUs');
Route::get('data-deletion-request', [FrontendController::class, 'datadeletionrequest'])->name('datadeletionrequest');
// Route::get('/home', [App\Http\Controllers\HomeController::class, 'index'])->name('home');
Route::prefix('admin')->name('admin.')->middleware(['auth','admin'])->group(function () {
    Route::get('edit-profile', [UserController::class, 'editprofile'])->name('editprofile');
    Route::post('update-profile', [UserController::class, 'updateprofile'])->name('updateprofile');
    Route::get('change-password', [UserController::class, 'change_password'])->name('changepassword');
    Route::post('update-password', [UserController::class, 'updatepassword'])->name('updatepassword');

    Route::get('dashboard', [DashboardController::class, 'index'])->name('dashboard');
    Route::get('', [DashboardController::class, 'index'])->name('dashboard');


    Route::prefix('roles')->name('roles.')->group(function () {
        Route::get('',[RoleController::class,'index'])->name('index');
        Route::post('store',[RoleController::class,'store'])->name('store');
        Route::get('edit/{id}',[RoleController::class,'edit'])->name('edit');
        Route::put('update/{id}',[RoleController::class,'update'])->name('update');
        Route::post('delete/{id}',[RoleController::class,'delete'])->name('delete');
        Route::get('assign-permission/{id}',[RoleController::class,'assignpermission'])->name('assignpermission');
        Route::post('allowpermission/{id}', [RoleController::class, 'allowpermissions'])->name('allowpermissions');
    });
    Route::prefix('permissions')->name('permissions.')->group(function () {
        Route::get('',[PermissionController::class,'index'])->name('index');
        Route::get('create',[PermissionController::class,'create'])->name('create');
        Route::post('store',[PermissionController::class,'store'])->name('store');
        Route::get('edit/{id}',[PermissionController::class,'edit'])->name('edit');
        Route::put('update/{id}',[PermissionController::class,'update'])->name('update');
        Route::post('delete/{id}',[PermissionController::class,'delete'])->name('delete');
    });
    Route::prefix('categories')->name('categories.')->group(function () {
        Route::get('',[CategoryController::class,'index'])->name('index');
        Route::post('store',[CategoryController::class,'store'])->name('store');
        Route::post('update',[CategoryController::class,'update'])->name('update');
        Route::post('delete/{id}',[CategoryController::class,'delete'])->name('delete');
    });
    Route::prefix('sub-categories')->name('subcategories.')->group(function () {
        Route::get('',[SubCategoryController::class,'index'])->name('index');
        Route::post('store',[SubCategoryController::class,'store'])->name('store');
        Route::get('edit/{id}',[SubCategoryController::class,'edit'])->name('edit');
        Route::post('update/{id}',[SubCategoryController::class,'update'])->name('update');
        Route::post('delete/{id}',[SubCategoryController::class,'delete'])->name('delete');
    });
    Route::prefix('gifts')->name('gifts.')->group(function () {
        Route::get('',[GiftController::class,'index'])->name('index');
        Route::get('create',[GiftController::class,'create'])->name('create');
        Route::post('store',[GiftController::class,'store'])->name('store');
        Route::get('edit/{id}',[GiftController::class,'edit'])->name('edit');
        Route::put('update/{id}',[GiftController::class,'update'])->name('update');
        Route::post('delete/{id}',[GiftController::class,'delete'])->name('delete');
    });
    Route::prefix('faqs')->name('faqs.')->group(function () {
        Route::get('',[FAQController::class,'index'])->name('index');
        Route::post('store',[FAQController::class,'store'])->name('store');
        Route::put('update',[FAQController::class,'update'])->name('update');
        Route::post('delete/{id}',[FAQController::class,'delete'])->name('delete');
    });
    Route::prefix('gateways')->name('gateways.')->group(function () {
        Route::get('',[GatewayController::class,'index'])->name('index');
        Route::post('store',[GatewayController::class,'store'])->name('store');
        Route::get('edit/{id}',[GatewayController::class,'edit'])->name('edit');
        Route::post('update/{id}',[GatewayController::class,'update'])->name('update');
        Route::post('delete/{id}',[GatewayController::class,'delete'])->name('delete');
    });
    Route::prefix('deposits')->name('deposits.')->group(function () {
        // Route::get('',[DepositController::class,'index'])->name('index');

    });
    Route::prefix('users')->name('users.')->group(function () {
        Route::get('',[ManageUserController::class,'index'])->name('index');
        Route::get('userlist',[ManageUserController::class,'userlist'])->name('userlist');
        Route::get('assign-roles/{id}',[ManageUserController::class,'assignroles'])->name('assignroles');
        Route::post('allow-roles/{id}',[ManageUserController::class,'allowroles'])->name('allowroles');
        Route::get('reports',[ManageUserController::class,'reports'])->name('reports');
        Route::post('report-action',[ManageUserController::class,'reportUserAction'])->name('reportaction');
        // Route::post('store',[ManageUserController::class,'store'])->name('store');
        // Route::get('edit/{id}',[ManageUserController::class,'edit'])->name('edit');
        // Route::put('update/{id}',[ManageUserController::class,'update'])->name('update');
        Route::get('delete/{id}',[ManageUserController::class,'delete'])->name('delete');
    });
    Route::get('term-conditions', [PolicyController::class, 'termConditions'])->name('termconditions');
    Route::post('term-conditions', [PolicyController::class, 'termConditionsUpdate'])->name('termconditions.update');
    Route::get('privacy-policy', [PolicyController::class, 'privaypolicy'])->name('privaypolicy');
    Route::post('privacy-policy', [PolicyController::class, 'privaypolicyUpdate'])->name('privacypolicy.update');
    Route::get('cookie-policy', [PolicyController::class, 'cookiepolicy'])->name('cookiepolicy');
    Route::post('cookie-policy', [PolicyController::class, 'cookiepolicyUpdate'])->name('cookiepolicy.update');
    Route::get('about', [PolicyController::class, 'about'])->name('about');
    Route::post('about', [PolicyController::class, 'aboutUpdate'])->name('about.update');
    Route::get('datadeletion', [PolicyController::class, 'datadeletion'])->name('datadeletion');
    Route::post('datadeletion', [PolicyController::class, 'datadeletionUpdate'])->name('datadeletion.update');

    Route::get('general-settings',[SettingController::class,'generalSetting'])->name('generalsettings');
    Route::post('update_setting', [SettingController::class, 'updateSetting'])->name('updatesetting');
    Route::post('uploadLogo', [SettingController::class, 'uploadLogo'])->name('upladLogo');
    Route::post('updateshare', [SettingController::class, 'updateShare'])->name('updateshare');    
    Route::prefix('withdraw-requests')->name('withdrawrequests.')->group(function () {
        Route::get('', [WithdrawRequestController::class, 'index'])->name('index');
        Route::get('list', [WithdrawRequestController::class, 'list'])->name('list');
        Route::get('view/{id}', [WithdrawRequestController::class, 'view'])->name('view');
        Route::post('action/{id}', [WithdrawRequestController::class, 'action'])->name('action');

    });
    Route::prefix('deposits')->name('deposits.')->group(function () {
        Route::get('', [DepositController::class, 'index'])->name('index');
        Route::get('list/', [DepositController::class, 'list'])->name('list');
        Route::get('details/{id}', [DepositController::class, 'details'])->name('details');
    });
    Route::prefix('spaces')->name('spaces.')->group(function () {
        Route::get('',[AdminSpaceController::class,'index'])->name('index');
        Route::get('list',[AdminSpaceController::class,'list'])->name('list');
        Route::get('details/{id}',[AdminSpaceController::class,'details'])->name('details');
        Route::get('report',[AdminSpaceController::class,'reportSpaces'])->name('report');
        Route::post('report-action',[AdminSpaceController::class,'reportSpacesaction'])->name('reportaction');
    });
    Route::prefix('posts')->name('posts.')->group(function () {
        Route::get('',[AdminPostController::class,'index'])->name('index');
        Route::get('list',[AdminPostController::class,'list'])->name('list');
        Route::get('delete/{id}',[AdminPostController::class,'delete'])->name('delete');
        Route::get('report',[AdminPostController::class,'reportPosts'])->name('reportPosts');
        Route::post('report-action',[AdminPostController::class,'reportSpacesaction'])->name('reportaction');
       
    });
    Route::get('export-user-data',[ManageUserController::class,'export'])->name('index');
    Route::get('export-deposits-pdf', [ManageUserController::class, 'export2']);
});

